@@ -226,6 +226,7 @@
         result)))
   #.sb-posix::eacces)
 
+#-(or (and darwin x86) win32)
 (deftest stat.1
   (let* ((stat (sb-posix:stat *test-directory*))
          (mode (sb-posix::stat-mode stat)))
@@ -233,7 +234,7 @@
     (logand mode (logior sb-posix::s-iread sb-posix::s-iwrite sb-posix::s-iexec)))
   #.(logior sb-posix::s-iread sb-posix::s-iwrite sb-posix::s-iexec))
 
-#-win32
+#-(or (and darwin x86) win32)
 (deftest stat.2
   (let* ((stat (sb-posix:stat "/"))
          (mode (sb-posix::stat-mode stat)))
@@ -254,7 +255,7 @@
     (< (- atime unix-now) 10))
   t)
 
-#-win32
+#-(or (and darwin x86) win32)
 (deftest stat.4
   (let* ((stat (sb-posix:stat (make-pathname :directory '(:absolute :up))))
          (mode (sb-posix::stat-mode stat)))
@@ -344,6 +345,7 @@
     (sb-posix:s-isreg mode))
   nil)
 
+#-(and darwin x86)
 (deftest stat-mode.2
   (with-stat-mode (mode *test-directory*)
     (sb-posix:s-isdir mode))
@@ -364,13 +366,13 @@
     (sb-posix:s-isfifo mode))
   nil)
 
-#-win32
+#-(or (and darwin x86) win32)
 (deftest stat-mode.6
   (with-stat-mode (mode *test-directory*)
     (sb-posix:s-issock mode))
   nil)
 
-#-win32
+#-(or (and darwin x86) win32)
 (deftest stat-mode.7
   (let ((link-pathname (make-pathname :name "stat-mode.7"
                                       :defaults *test-directory*)))
@@ -382,6 +384,7 @@
       (ignore-errors (sb-posix:unlink link-pathname))))
   t)
 
+#-(and darwin x86)
 (deftest stat-mode.8
   (let ((pathname (make-pathname :name "stat-mode.8"
                                  :defaults *test-directory*)))
@@ -429,14 +432,14 @@
   #+win32
   #.sb-posix:eacces)
 
-#-(or (and (or x86-64 arm64) (or linux sunos)) win32)
+#-(or (and (or x86-64 arm64 alpha) (or linux sunos)) win32)
 (deftest fcntl.1
   (let ((fd (sb-posix:open "/dev/null" sb-posix::o-nonblock)))
     (= (sb-posix:fcntl fd sb-posix::f-getfl) sb-posix::o-nonblock))
   t)
 ;; On AMD64/Linux O_LARGEFILE is always set, even though the whole
 ;; flag makes no sense.
-#+(and (or x86-64 arm64) (or linux sunos))
+#+(and (or x86-64 arm64 alpha) (or linux sunos))
 (deftest fcntl.1
   (let ((fd (sb-posix:open "/dev/null" sb-posix::o-nonblock)))
     (/= 0 (logand (sb-posix:fcntl fd sb-posix::f-getfl)
@@ -535,6 +538,7 @@
         (sb-posix:closedir dir))))
   nil)
 
+#-(and darwin x86)
 (deftest readdir.1
   (let ((dir (sb-posix:opendir "/")))
     (unwind-protect
@@ -662,7 +666,7 @@
     (plusp (sb-posix:time))
   t)
 
-#-(or win32)
+#-(or (and darwin x86) win32)
 (deftest utimes.1
     (let ((file (merge-pathnames #p"utimes.1" *test-directory*))
           (atime (random (1- (expt 2 31))))
