@@ -69,7 +69,7 @@
   (:import-from #:sb-di #:valid-lisp-pointer-p)
   (:import-from #:sb-disassem #:dstate-inst-properties)
   (:import-from #:sb-x86-64-asm
-                #:lock #:x66 #:rex #:+rex-r+
+                #:lock #:x66 #:rex #:+rex-b+
                 #:inst-operand-size
                 #:machine-ea-p
                 #:machine-ea-base
@@ -130,14 +130,12 @@
 
 (defconstant +state-initial+         1)
 (defconstant +state-profiler+        2)
-(defconstant +state-loaded-free-ptr+ 3)
-(defconstant +state-bumped-free-ptr+ 4)
-(defconstant +state-tested-free-ptr+ 5)
-(defconstant +state-jumped+          6)
-(defconstant +state-stored-free-ptr+ 7)
-;; moved-result is a spurious instruction and I'd like to eliminate it.
-;; But this code has to recognize it meanwhile.
-;(defconstant +state-moved-result+    8)
+(defconstant +state-begin-pa+        3)
+(defconstant +state-loaded-free-ptr+ 4)
+(defconstant +state-bumped-free-ptr+ 5)
+(defconstant +state-tested-free-ptr+ 6)
+(defconstant +state-jumped+          7)
+(defconstant +state-stored-free-ptr+ 8)
 (defconstant +state-descriptorized+  9)
 (defconstant +state-widetagged+     10)
 (defconstant +state-trampoline-arg+ 11)
@@ -165,6 +163,8 @@
 ;;; So we disassemble just enough instructions at PC to deduce
 ;;; what type of object is being allocated.
 (defun infer-type (pc component)
+  (declare (ignorable pc component))
+  #+(and x86-64 sb-thread)
   (let* ((seg (sb-disassem::%make-segment
                :sap-maker (lambda () (error "Bad sap maker"))
                :virtual-location 0))
@@ -232,14 +232,23 @@
                        (null (machine-ea-index ea))
                        (eq (machine-ea-disp ea)
                            (+ profiler-index sb-vm:n-word-bytes))))
-                 ((eq opcode '|push|)
-                  ;; Known huge allocation goes straight to C call
-                  (setq size (ldb (byte 32 8) dchunk))
-                  (advance +state-trampoline-arg+))
                  (t
-                  (advance-if (and (eq opcode 'mov) free-ptr-p)
-                              +state-loaded-free-ptr+)
-                  (setq orig-free-ptr-reg (regrm-inst-reg dchunk dstate)))))
+                  (advance-if (and (eq opcode 'mov)
+                                   (let ((rm (regrm-inst-r/m 0 dstate)))
+                                     (eq (machine-ea-base rm) thread-base-reg)
+                                     (eq (machine-ea-disp rm)
+                                         (ash sb-vm::thread-pseudo-atomic-bits-slot
+                                              sb-vm:word-shift))))
+                              +state-begin-pa+))))
+               (#.+state-begin-pa+
+                (cond ((eq opcode '|push|)
+                       ;; Known huge allocation goes straight to C call
+                       (setq size (ldb (byte 32 8) dchunk))
+                       (advance +state-trampoline-arg+))
+                      (t
+                       (advance-if (and (eq opcode 'mov) free-ptr-p)
+                                   +state-loaded-free-ptr+)
+                       (setq orig-free-ptr-reg (regrm-inst-reg dchunk dstate)))))
                (#.+state-loaded-free-ptr+
                 (case opcode
                  ;; Variable-size alloc can either use LEA or ADD to compute the
@@ -348,7 +357,7 @@
                (#.+state-called+
                 (cond ((and (eq opcode '|pop|) (<= #x58 opcode-byte #x5F))
                        (setq target-reg
-                             (+ (if (logtest +rex-r+ (dstate-inst-properties dstate))
+                             (+ (if (logtest +rex-b+ (dstate-inst-properties dstate))
                                     8 0)
                                 (- opcode-byte #x58)))
                        (advance +state-result-popped+))
@@ -530,16 +539,22 @@
            (format t "~19@t===========~%~19@t~11d~%" sum-bytes))
           (t
            (format t " =======  ===========~%~6,1,2f   ~12d~%"
-                   sum-pct sum-bytes)))))
+                   sum-pct sum-bytes)))
+    sum-bytes))
 
+;;; Call FUN and return the exact number of bytes it (an all descendant
+;;; calls) allocated, provided that they were instrumented for precise
+;;; cons profiling.
 (defun aprof-run (fun)
   (aprof-reset)
   (patch-fixups)
-  (unwind-protect
-       (progn (aprof-start) (funcall fun))
-    (aprof-stop)
-    (aprof-show)
-    (terpri)))
+  (let (nbytes)
+    (unwind-protect
+         (progn (aprof-start) (funcall fun))
+      (aprof-stop)
+      (setq nbytes (aprof-show))
+      (terpri))
+    nbytes))
 
 ;;;;
 
