@@ -9,14 +9,16 @@
 
 ;;;; FIXUP-CODE-OBJECT
 
+(defconstant-eqx +fixup-kinds+ #(:absolute :cond-branch :uncond-branch)
+  #'equalp)
 (!with-bigvec-or-sap
-(defun fixup-code-object (code offset fixup kind)
+(defun fixup-code-object (code offset fixup kind flavor)
   (declare (type index offset))
-  (unless (zerop (rem offset 4))
+  (declare (ignore flavor))
+  (unless (zerop (rem offset sb!assem:+inst-alignment-bytes+))
     (error "Unaligned instruction?  offset=#x~X." offset))
-  (without-gcing
-    (let ((sap (code-instructions code)))
-      (ecase kind
+  (let ((sap (code-instructions code)))
+    (ecase kind
         (:absolute
          (setf (sap-ref-word sap offset) fixup))
         (:cond-branch
@@ -24,7 +26,8 @@
                (ash (- fixup (+ (sap-int sap) offset)) -2)))
         (:uncond-branch
          (setf (ldb (byte 26 0) (sap-ref-32 sap offset))
-               (ash (- fixup (+ (sap-int sap) offset)) -2))))))))
+               (ash (- fixup (+ (sap-int sap) offset)) -2)))))
+  nil))
 
 ;;;; "Sigcontext" access functions, cut & pasted from sparc-vm.lisp,
 ;;;; then modified for ARM.
@@ -78,9 +81,26 @@
          (error-number (ldb (byte 8 13) instruction))
          (trap-number (ldb (byte 8 5) instruction)))
     (declare (type system-area-pointer pc))
-    (values error-number
-            (if (= trap-number invalid-arg-count-trap)
-                '(#.arg-count-sc)
-                (sb!kernel::decode-internal-error-args (sap+ pc 4) error-number))
-            trap-number)))
+    (if (= trap-number invalid-arg-count-trap)
+        (values error-number '(#.arg-count-sc) trap-number)
+        (sb!kernel::decode-internal-error-args (sap+ pc 4) trap-number error-number))))
 ) ; end PROGN
+
+;;; Undo the effects of XEP-ALLOCATE-FRAME
+;;; and point PC to FUNCTION
+#-sb-xc-host
+(defun context-call-function (context function &optional arg-count)
+  (with-pinned-objects (function)
+    (with-pinned-context-code-object (context)
+      (let* ((fun-addr (get-lisp-obj-address function))
+             (entry (+ (sap-ref-word (int-sap fun-addr)
+                                     (- (ash simple-fun-self-slot word-shift)
+                                        fun-pointer-lowtag))
+                       (- (ash simple-fun-code-offset word-shift)
+                          fun-pointer-lowtag))))
+        (when arg-count
+          (setf (context-register context nargs-offset)
+                (get-lisp-obj-address arg-count)))
+        (setf (context-register context lexenv-offset) fun-addr
+              (context-register context lr-offset) entry)
+        (set-context-pc context entry)))))
