@@ -165,15 +165,13 @@ invoked. In that case it will store into PLACE and start over."
       (setf (info :variable :kind name) :macro)
       (setf (info :variable :macro-expansion name) expansion))
      (t
-      (error 'simple-program-error
-             :format-control "Symbol ~S is already defined as ~A."
-             :format-arguments (list name
-                                     (case kind
-                                       (:alien "an alien variable")
-                                       (:constant "a constant")
-                                       (:special "a special variable")
-                                       (:global "a global variable")
-                                       (t kind)))))))
+      (%program-error "Symbol ~S is already defined as ~A."
+                      name (case kind
+                             (:alien "an alien variable")
+                             (:constant "a constant")
+                             (:special "a special variable")
+                             (:global "a global variable")
+                             (t kind))))))
   name)
 
 ;;;; DEFINE-COMPILER-MACRO
@@ -182,9 +180,8 @@ invoked. In that case it will store into PLACE and start over."
   "Define a compiler-macro for NAME."
   (legal-fun-name-or-type-error name)
   (when (and (symbolp name) (special-operator-p name))
-    (error 'simple-program-error
-           :format-control "cannot define a compiler-macro for a special operator: ~S"
-           :format-arguments (list name)))
+    (%program-error "cannot define a compiler-macro for a special operator: ~S"
+                    name))
   ;; DEBUG-NAME is called primarily for its side-effect of asserting
   ;; that (COMPILER-MACRO-FUNCTION x) is not a legal function name.
   (let ((def (make-macro-lambda (sb!c::debug-name 'compiler-macro name)
@@ -192,7 +189,7 @@ invoked. In that case it will store into PLACE and start over."
                                 :accessor 'sb!c::compiler-macro-args)))
     `(progn
           (eval-when (:compile-toplevel)
-           (sb!c::%compiler-defmacro :compiler-macro-function ',name t))
+           (sb!c::%compiler-defmacro :compiler-macro-function ',name))
           (eval-when (:compile-toplevel :load-toplevel :execute)
            (sb!c::%define-compiler-macro ',name ,def)))))
 
@@ -309,7 +306,7 @@ invoked. In that case it will store into PLACE and start over."
                             :format-control
                             "~@<~IBad ~S clause:~:@_  ~S~:@_~S allowed as the key ~
                            designator only in the final otherwise-clause, not in a ~
-                           normal-clause. Use (~S) instead, or move the clause the ~
+                           normal-clause. Use (~S) instead, or move the clause to the ~
                            correct position.~:@>"
                             :format-arguments (list 'case case keyoid keyoid)
                             :references `((:ansi-cl :macro case))))
@@ -502,7 +499,9 @@ invoked. In that case it will store into PLACE and start over."
   ;; optional dispatch mechanism for the M-V-B gets increasingly
   ;; hairy.
   (let ((val (and (sb!xc:constantp n env) (constant-form-value n env))))
-    (if (and (integerp val) (<= 0 val 10)) ; Arbitrary limit.
+    (if (and (integerp val) (<= 0 val (or #!+(or x86-64 arm64) ;; better DEFAULT-UNKNOWN-VALUES
+                                          1000
+                                          10))) ; Arbitrary limit.
         (let ((dummy-list (make-gensym-list val))
               (keeper (sb!xc:gensym "KEEPER")))
           `(multiple-value-bind (,@dummy-list ,keeper) ,form
@@ -555,3 +554,28 @@ invoked. In that case it will store into PLACE and start over."
     (let ((fn-sym (sb!xc:gensym))) ; for ONCE-ONLY-ish purposes
       (values `((,fn-sym (%coerce-callable-to-fun ,funarg)))
               `(sb!c::%funcall ,fn-sym . ,arg-forms)))))
+
+;;; Ordinarily during self-build, nothing would need this macro except the
+;;; calls in src/code/list, and src/code/seq.
+;;; However, if cons profiling is enbled, then all calls to COPY-LIST
+;;; transform into the macro, so it must be available early.
+(sb!xc:defmacro copy-list-macro (input &key check-proper-list)
+  ;; Unless CHECK-PROPER-LIST is true, the list is copied correctly
+  ;; even if the list is not terminated by NIL. The new list is built
+  ;; by CDR'ing SPLICE which is always at the tail of the new list.
+  (with-unique-names (orig copy splice cell)
+    ;; source transform gave input the ONCE-ONLY treatment already.
+    `(when ,input
+       (let ((,copy (list (car ,input))))
+         (do ((,orig (cdr ,input) (cdr ,orig))
+              (,splice ,copy
+                       (let ((,cell (list (car ,orig))))
+                         (rplacd (truly-the cons ,splice) ,cell)
+                         ,cell)))
+           (,@(if check-proper-list
+                  `((endp ,orig))
+                  `((atom ,orig)
+                    ;; always store the CDR even if NIL. A blind write
+                    ;; is cheaper than a branch around a write.
+                    (rplacd (truly-the cons ,splice) ,orig)))
+            ,copy))))))
