@@ -198,9 +198,9 @@ distinct from the global value. Can also be SETF."
 
 (defun symbol-plist (symbol)
   "Return SYMBOL's property list."
-  #!+symbol-info-vops
-  (symbol-plist symbol) ; VOP translates it
-  #!-symbol-info-vops
+  #!+(vop-translates cl:symbol-plist)
+  (symbol-plist symbol)
+  #!-(vop-translates cl:symbol-plist)
   (let ((list (car (truly-the list (symbol-info symbol))))) ; a white lie
     ;; Just ensure the result is not a fixnum, and we're done.
     (if (fixnump list) nil list)))
@@ -332,7 +332,7 @@ distinct from the global value. Can also be SETF."
                (char= (char name 0) #\*)
                (char= (char name (1- (length name))) #\*)))
       #!+immobile-symbols t ; always place them there
-      (truly-the (values symbol) (%primitive sb!vm::alloc-immobile-symbol name))
+      (truly-the (values symbol) (sb!vm::make-immobile-symbol name))
       (sb!vm::%%make-symbol name)))
 
 (defun get (symbol indicator &optional (default nil))
