@@ -116,19 +116,14 @@ This is SETFable."
   ;;        The alien is allocated on the heap, and has infinite extent. The alien
   ;;        is allocated at load time, so the same piece of memory is used each time
   ;;        this form executes.
-  (/noshow "entering WITH-ALIEN" bindings)
-  (let (#!+c-stack-is-control-stack
-        bind-alien-stack-pointer)
+  (let (bind-alien-stack-pointer)
     (with-auxiliary-alien-types env
       (dolist (binding (reverse bindings))
-        (/noshow binding)
         (destructuring-bind
             (symbol type &optional opt1 (opt2 nil opt2p))
             binding
-          (/noshow symbol type opt1 opt2)
           (let* ((alien-type (parse-alien-type type env))
                  (datap (not (alien-fun-type-p alien-type))))
-            (/noshow alien-type)
             (multiple-value-bind (allocation initial-value)
                 (if opt2p
                     (values opt1 opt2)
@@ -139,7 +134,6 @@ This is SETFable."
                        (values opt1 nil))
                       (t
                        (values :local opt1))))
-              (/noshow allocation initial-value)
               (setf body
                     (ecase allocation
                       #+nil
@@ -155,14 +149,12 @@ This is SETFable."
                                    `((setq ,symbol ,initial-value)))
                                ,@body)))))
                       (:extern
-                       (/noshow0 ":EXTERN case")
                        `((symbol-macrolet
                              ((,symbol
                                 (%alien-value
                                  (foreign-symbol-sap ,initial-value ,datap) 0 ,alien-type)))
                            ,@body)))
                       (:local
-                       (/noshow0 ":LOCAL case")
                        (let* ((var (sb!xc:gensym "VAR"))
                               (initval (if initial-value (sb!xc:gensym "INITVAL")))
                               (info (make-local-alien-info :type alien-type))
@@ -177,34 +169,21 @@ This is SETFable."
                                     `((let ((,initval ,initial-value))
                                         ,@inner-body))
                                     inner-body)))
-                         (/noshow var initval info)
-                         #!+c-stack-is-control-stack
-                         (progn (setf bind-alien-stack-pointer t)
-                                `((let ((,var (make-local-alien ',info)))
-                                    ,@body-forms)))
-                         ;; FIXME: This version is less efficient then it needs to be, since
-                         ;; it could just save and restore the number-stack pointer once,
-                         ;; instead of doing multiple decrements if there are multiple bindings.
-                         #!-c-stack-is-control-stack
+                         (setf bind-alien-stack-pointer t)
                          `((let ((,var (make-local-alien ',info)))
-                             (multiple-value-prog1
-                                 (progn ,@body-forms)
-                               ;; No need for unwind protect here, since
-                               ;; allocation involves modifying NSP, and
-                               ;; NSP is saved and restored during NLX.
-                               ;; And in non-transformed case it
-                               ;; performs finalization.
-                               (dispose-local-alien ',info ,var))))))))))))
-      (/noshow "revised" body)
+                             ,@body-forms))))))))))
       (verify-local-auxiliaries-okay)
-      (/noshow0 "back from VERIFY-LOCAL-AUXILIARIES-OK, returning")
       `(symbol-macrolet ((&auxiliary-type-definitions&
                            ,(append *new-auxiliary-types*
                                     (auxiliary-type-definitions env))))
          ,@(cond
-             #!+c-stack-is-control-stack
              (bind-alien-stack-pointer
-              `((let ((sb!vm::*alien-stack-pointer* sb!vm::*alien-stack-pointer*))
+              ;; The LET IR1-translator will actually turn this into
+              ;; RESTORING-NSP on #-c-stack-is-control-stack to avoid
+              ;; expanding into non-standard special forms.
+              ;; And the LET has to look exactly like this, not LET*
+              ;; and no other bindings.
+              `((let ((sb!c:*alien-stack-pointer* sb!c:*alien-stack-pointer*))
                   ,@body)))
              (t
               body))))))
@@ -328,11 +307,6 @@ Examples:
         (malloc-error bytes (get-errno))
         sap)))
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defvar *saved-fp-and-pcs* nil)
-  ;; Can't use DECLAIM since always-bound is a non-standard declaration
-  (sb!xc:proclaim '(sb!ext:always-bound *saved-fp-and-pcs*)))
-
 #!+c-stack-is-control-stack
 (declaim (inline invoke-with-saved-fp-and-pc))
 ;;; On :c-stack-is-control-stack platforms, this DEFUN must appear prior to the
@@ -413,8 +387,7 @@ null byte."
 (defun slot (alien slot)
   "Extract SLOT from the Alien STRUCT or UNION ALIEN. May be set with SETF."
   (declare (type alien-value alien)
-           (type symbol slot)
-           (optimize (inhibit-warnings 3)))
+           (type symbol slot))
   (let ((type (alien-value-type alien)))
     (etypecase type
       (alien-pointer-type
@@ -430,8 +403,7 @@ null byte."
 ;;; this when it can't figure out anything better.
 (defun %set-slot (alien slot value)
   (declare (type alien-value alien)
-           (type symbol slot)
-           (optimize (inhibit-warnings 3)))
+           (type symbol slot))
   (let ((type (alien-value-type alien)))
     (etypecase type
       (alien-pointer-type
@@ -446,8 +418,7 @@ null byte."
 ;;; Compute the address of the specified slot and return a pointer to it.
 (defun %slot-addr (alien slot)
   (declare (type alien-value alien)
-           (type symbol slot)
-           (optimize (inhibit-warnings 3)))
+           (type symbol slot))
   (let ((type (alien-value-type alien)))
     (etypecase type
       (alien-pointer-type
@@ -507,8 +478,7 @@ null byte."
    as the indices of the array element to access. If a pointer, one index can
    optionally be specified, giving the equivalent of C pointer arithmetic."
   (declare (type alien-value alien)
-           (type list indices)
-           (optimize (inhibit-warnings 3)))
+           (type list indices))
   (multiple-value-bind (target-type offset) (deref-guts alien indices)
     (%alien-value (alien-value-sap alien)
                   offset
@@ -516,8 +486,7 @@ null byte."
 
 (defun %set-deref (alien value &rest indices)
   (declare (type alien-value alien)
-           (type list indices)
-           (optimize (inhibit-warnings 3)))
+           (type list indices))
   (multiple-value-bind (target-type offset) (deref-guts alien indices)
     (setf (%alien-value (alien-value-sap alien)
                         offset
@@ -526,8 +495,7 @@ null byte."
 
 (defun %deref-addr (alien &rest indices)
   (declare (type alien-value alien)
-           (type list indices)
-           (optimize (inhibit-warnings 3)))
+           (type list indices))
   (multiple-value-bind (target-type offset) (deref-guts alien indices)
     (%sap-alien (sap+ (alien-value-sap alien) (/ offset sb!vm:n-byte-bits))
                 (make-alien-pointer-type :to target-type))))
@@ -535,23 +503,20 @@ null byte."
 ;;;; accessing heap alien variables
 
 (defun %heap-alien (info)
-  (declare (type heap-alien-info info)
-           (optimize (inhibit-warnings 3)))
+  (declare (type heap-alien-info info))
   (%alien-value (heap-alien-info-sap info)
                 0
                 (heap-alien-info-type info)))
 
 (defun %set-heap-alien (info value)
-  (declare (type heap-alien-info info)
-           (optimize (inhibit-warnings 3)))
+  (declare (type heap-alien-info info))
   (setf (%alien-value (heap-alien-info-sap info)
                       0
                       (heap-alien-info-type info))
         value))
 
 (defun %heap-alien-addr (info)
-  (declare (type heap-alien-info info)
-           (optimize (inhibit-warnings 3)))
+  (declare (type heap-alien-info info))
   (%sap-alien (heap-alien-info-sap info)
               (make-alien-pointer-type :to (heap-alien-info-type info))))
 
@@ -609,14 +574,6 @@ null byte."
   (unless (local-alien-info-force-to-memory-p info)
     (error "~S isn't forced to memory. Something went wrong." alien))
   alien)
-
-;; It's not mandatory that this function not exist for x86[-64],
-;; however for sanity, it should not, because no call to it can occur.
-#!-(or x86 x86-64)
-(defun dispose-local-alien (info alien)
-  (declare (ignore info))
-  (cancel-finalization alien)
-  (free-alien alien))
 
 ;;;; the CAST macro
 
@@ -628,8 +585,7 @@ null byte."
 (defun %cast (alien target-type)
   (declare (type alien-value alien)
            (type alien-type target-type)
-           (optimize (safety 2))
-           (optimize (inhibit-warnings 3)))
+           (optimize (safety 2)))
   (if (or (alien-pointer-type-p target-type)
           (alien-array-type-p target-type)
           (alien-fun-type-p target-type))
@@ -859,277 +815,5 @@ way that the argument is passed.
   (when (alien-value-p object)
     (alien-subtype-p (alien-value-type object) type)))
 
-;;;; ALIEN CALLBACKS
-;;;;
-;;;; See "Foreign Linkage / Callbacks" in the SBCL Internals manual.
-
-(defvar *alien-callback-info* nil
-  "Maps SAPs to corresponding CALLBACK-INFO structures: contains all the
-information we need to manipulate callbacks after their creation. Used for
-changing the lisp-side function they point to, invalidation, etc.")
-
-(defstruct (callback-info
-            (:copier nil))
-  specifier
-  function ; NULL if invalid
-  wrapper
-  index)
-
-(defun callback-info-key (info)
-  (cons (callback-info-specifier info) (callback-info-function info)))
-
-(defun alien-callback-info (alien)
-  (cdr (assoc (alien-sap alien) *alien-callback-info* :test #'sap=)))
-
-(define-load-time-global *alien-callbacks* (make-hash-table :test #'equal)
-  "Cache of existing callback SAPs, indexed with (SPECIFER . FUNCTION). Used for
-memoization: we don't create new callbacks if one pointing to the correct
-function with the same specifier already exists.")
-
-(define-load-time-global *alien-callback-wrappers* (make-hash-table :test #'equal)
-  "Cache of existing lisp wrappers, indexed with SPECIFER. Used for memoization:
-we don't create new wrappers if one for the same specifier already exists.")
-
-(define-load-time-global *alien-callback-trampolines*
-    (make-array 32 :fill-pointer 0 :adjustable t)
-  "Lisp trampoline store: assembler wrappers contain indexes to this, and
-ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
-
-(defun %alien-callback-sap (specifier result-type argument-types function wrapper
-                            &optional call-type)
-  (declare #!-x86 (ignore call-type))
-  (ensure-gethash
-   (list specifier function) *alien-callbacks*
-   (let* ((index (fill-pointer *alien-callback-trampolines*))
-          ;; Aside from the INDEX this is known at
-          ;; compile-time, which could be utilized by
-          ;; having the two-stage assembler tramp &
-          ;; wrapper mentioned in [1] above: only the
-          ;; per-function tramp would need assembler at
-          ;; runtime. Possibly we could even pregenerate
-          ;; the code and just patch the index in later.
-          (assembler-wrapper
-           (alien-callback-assembler-wrapper
-            index result-type argument-types
-            #!+x86
-            (if (eq call-type :stdcall)
-                (ceiling
-                 (apply #'+
-                        (mapcar 'alien-type-word-aligned-bits
-                                argument-types))
-                 8)
-                0))))
-     (vector-push-extend
-      (alien-callback-lisp-trampoline wrapper function)
-      *alien-callback-trampolines*)
-     ;; Assembler-wrapper is static, so sap-taking is safe.
-     (let ((sap (vector-sap assembler-wrapper)))
-       (push (cons sap (make-callback-info :specifier specifier
-                                           :function function
-                                           :wrapper wrapper
-                                           :index index))
-             *alien-callback-info*)
-       sap))))
-
-(defun alien-callback-lisp-trampoline (wrapper function)
-  (declare (function wrapper) (optimize speed))
-  (lambda (args-pointer result-pointer)
-    (funcall wrapper args-pointer result-pointer function)))
-
-(defun alien-callback-lisp-wrapper-lambda (specifier result-type argument-types env)
-  (let* ((arguments (make-gensym-list (length argument-types)))
-         (argument-names arguments)
-         (argument-specs (cddr specifier)))
-    `(lambda (args-pointer result-pointer function)
-       ;; KLUDGE: the SAP shouldn't be consed but they are, don't
-       ;; bother anyone about that sad fact
-       (declare (muffle-conditions compiler-note))
-       ;; FIXME: the saps are not gc safe
-       (let ((args-sap (int-sap
-                        (sb!kernel:get-lisp-obj-address args-pointer)))
-             (res-sap (int-sap
-                       (sb!kernel:get-lisp-obj-address result-pointer))))
-         (declare (ignorable args-sap res-sap))
-         (with-alien
-             ,(loop
-                 with offset = 0
-                 for spec in argument-specs
-                 ;; KLUDGE: At least one platform requires additional
-                 ;; alignment beyond a single machine word for certain
-                 ;; arguments.  Accept an additional delta (for the
-                 ;; alignment) to apply to subsequent arguments to
-                 ;; account for the alignment gaps as a secondary
-                 ;; value, so that we don't have to update unaffected
-                 ;; backends.
-                 for (accessor-form alignment)
-                   = (multiple-value-list
-                      (alien-callback-accessor-form spec 'args-sap offset))
-                 collect `(,(pop argument-names) ,spec
-                            :local ,accessor-form)
-                 do (incf offset (+ (alien-callback-argument-bytes spec env)
-                                    (or alignment 0))))
-           ,(flet ((store (spec real-type)
-                          (if spec
-                              `(setf (deref (sap-alien res-sap (* ,spec)))
-                                     ,(if real-type
-                                          `(the ,real-type
-                                             (funcall function ,@arguments))
-                                          `(funcall function ,@arguments)))
-                              `(funcall function ,@arguments))))
-                  (cond ((alien-void-type-p result-type)
-                         (store nil nil))
-                        ((alien-integer-type-p result-type)
-                         ;; Integer types should be padded out to a full
-                         ;; register width, to comply with most ABI calling
-                         ;; conventions, but should be typechecked on the
-                         ;; declared type width, hence the following:
-                         (if (alien-integer-type-signed result-type)
-                             (store `(signed
-                                      ,(alien-type-word-aligned-bits result-type))
-                                    `(signed-byte ,(alien-type-bits result-type)))
-                             (store
-                              `(unsigned
-                                ,(alien-type-word-aligned-bits result-type))
-                              `(unsigned-byte ,(alien-type-bits result-type)))))
-                        (t
-                         (store (unparse-alien-type result-type) nil))))))
-       (values))))
-
-(defun invalid-alien-callback (&rest arguments)
-  (declare (ignore arguments))
-  (error "Invalid alien callback called."))
-
-(defun parse-callback-specification (result-type lambda-list)
-  (values
-   `(function ,result-type ,@(mapcar #'second lambda-list))
-   (mapcar #'first lambda-list)))
-
-(defun parse-alien-ftype (specifier env)
-  (destructuring-bind (function result-type &rest argument-types)
-      specifier
-    (aver (eq 'function function))
-    (multiple-value-bind (bare-result-type calling-convention)
-        (typecase result-type
-          ((cons calling-convention *)
-             (values (second result-type) (first result-type)))
-          (t result-type))
-      (values (let ((*values-type-okay* t))
-                (parse-alien-type bare-result-type env))
-              (mapcar (lambda (spec)
-                        (parse-alien-type spec env))
-                      argument-types)
-              calling-convention))))
-
 (defun alien-void-type-p (type)
   (and (alien-values-type-p type) (not (alien-values-type-values type))))
-
-(defun alien-type-word-aligned-bits (type)
-  (align-offset (alien-type-bits type) sb!vm:n-word-bits))
-
-(defun alien-callback-argument-bytes (spec env)
-  (let ((type (parse-alien-type spec env)))
-    (if (or (alien-integer-type-p type)
-            (alien-float-type-p type)
-            (alien-pointer-type-p type)
-            (alien-system-area-pointer-type-p type))
-        (ceiling (alien-type-word-aligned-bits type) sb!vm:n-byte-bits)
-        (error "Unsupported callback argument type: ~A" type))))
-
-(defun enter-alien-callback (index return arguments)
-  (funcall (aref *alien-callback-trampolines* index)
-           return
-           arguments))
-
-;;;; interface (not public, yet) for alien callbacks
-
-(let ()
-(defmacro alien-callback (specifier function &environment env)
-  "Returns an alien-value with of alien ftype SPECIFIER, that can be passed to
-an alien function as a pointer to the FUNCTION. If a callback for the given
-SPECIFIER and FUNCTION already exists, it is returned instead of consing a new
-one."
-  ;; Pull out as much work as is convenient to macro-expansion time, specifically
-  ;; everything that can be done given just the SPECIFIER and ENV.
-  (multiple-value-bind (result-type argument-types call-type)
-      (parse-alien-ftype specifier env)
-    `(%sap-alien
-      (%alien-callback-sap ',specifier ',result-type ',argument-types
-                           ,function
-                           (ensure-gethash
-                            ',specifier *alien-callback-wrappers*
-                            ,(alien-callback-lisp-wrapper-lambda
-                              specifier result-type argument-types env))
-                           ,call-type)
-      ',(parse-alien-type specifier env)))))
-
-(defun alien-callback-p (alien)
-  "Returns true if the alien is associated with a lisp-side callback,
-and a secondary return value of true if the callback is still valid."
-  (let ((info (alien-callback-info alien)))
-    (when info
-      (values t (and (callback-info-function info) t)))))
-
-(defun alien-callback-function (alien)
-  "Returns the lisp function designator associated with the callback."
-  (let ((info (alien-callback-info alien)))
-    (when info
-      (callback-info-function info))))
-
-(defun (setf alien-callback-function) (function alien)
-  "Changes the lisp function designated by the callback."
-  (let ((info (alien-callback-info alien)))
-    (unless info
-      (error "Not an alien callback: ~S" alien))
-    ;; sap cache
-    (let ((key (callback-info-key info)))
-      (remhash key *alien-callbacks*)
-      (setf (gethash key *alien-callbacks*) (alien-sap alien)))
-    ;; trampoline
-    (setf (aref *alien-callback-trampolines* (callback-info-index info))
-          (alien-callback-lisp-trampoline (callback-info-wrapper info) function))
-    ;; metadata
-    (setf (callback-info-function info) function)
-    function))
-
-(defun invalidate-alien-callback (alien)
-  "Invalidates the callback designated by the alien, if any, allowing the
-associated lisp function to be GC'd, and causing further calls to the same
-callback signal an error."
-  (let ((info (alien-callback-info alien)))
-    (when (and info (callback-info-function info))
-      ;; sap cache
-      (remhash (callback-info-key info) *alien-callbacks*)
-      ;; trampoline
-      (setf (aref *alien-callback-trampolines* (callback-info-index info))
-            #'invalid-alien-callback)
-      ;; metadata
-      (setf (callback-info-function info) nil)
-      t)))
-
-;;; FIXME: This call assembles a new callback for every closure,
-;;; which sucks hugely. ...not that I can think of an obvious
-;;; solution. Possibly maybe we could write a generalized closure
-;;; callback analogous to closure_tramp, and share the actual wrapper?
-;;;
-;;; For lambdas that result in simple-funs we get the callback from
-;;; the cache on subsequent calls.
-(let ()
-(defmacro alien-lambda (result-type typed-lambda-list &body forms)
-  (multiple-value-bind (specifier lambda-list)
-      (parse-callback-specification result-type typed-lambda-list)
-    `(alien-callback ,specifier (lambda ,lambda-list ,@forms))))
-
-;;; FIXME: Should subsequent (SETF FDEFINITION) affect the callback or not?
-;;; What about subsequent DEFINE-ALIEN-CALLBACKs? My guess is that changing
-;;; the FDEFINITION should invalidate the callback, and redefining the
-;;; callback should change existing callbacks to point to the new defintion.
-(defmacro define-alien-callback (name result-type typed-lambda-list &body forms)
-  "Defines #'NAME as a function with the given body and lambda-list, and NAME as
-the alien callback for that function with the given alien type."
-  (declare (symbol name))
-  (multiple-value-bind (specifier lambda-list)
-      (parse-callback-specification result-type typed-lambda-list)
-    `(progn
-       (defun ,name ,lambda-list ,@forms)
-       (defparameter ,name (alien-callback ,specifier #',name)))))
-)
