@@ -14,8 +14,7 @@
 
 ;;; various environment inquiries
 
-(defvar *features*
-  '#.(sort (copy-list sb-cold:*shebang-features*) #'string<)
+(!defvar *features* '#.sb!xc:*features*
   "a list of symbols that describe features provided by the
    implementation")
 
@@ -133,7 +132,7 @@ the file system."
 (defun %defglobal (name value boundp doc docp source-location)
   (%compiler-defglobal name :always-bound value (not boundp))
   (when docp
-    (setf (fdocumentation name 'variable) doc))
+    (setf (documentation name 'variable) doc))
   (when source-location
     (setf (info :source-location :variable name) source-location))
   name)
@@ -142,7 +141,7 @@ the file system."
   (%compiler-defvar var)
   (set var val)
   (when docp
-    (setf (fdocumentation var 'variable) doc))
+    (setf (documentation var 'variable) doc))
   (when source-location
     (setf (info :source-location :variable var) source-location))
   var)
@@ -153,16 +152,16 @@ the file system."
              (not (boundp var)))
     (set var val))
   (when docp
-    (setf (fdocumentation var 'variable) doc))
+    (setf (documentation var 'variable) doc))
   (when source-location
     (setf (info :source-location :variable var) source-location))
   var)
 
-(defun %defun (name def &optional inline-lambda)
+(defun %defun (name def &optional inline-lambda dxable-args)
   (declare (type function def))
   ;; should've been checked by DEFMACRO DEFUN
   (aver (legal-fun-name-p name))
-  (sb!c:%compiler-defun name inline-lambda nil)
+  (sb!c:%compiler-defun name nil inline-lambda dxable-args)
   (when (fboundp name)
     (warn 'redefinition-with-defun :name name :new-function def))
   (setf (fdefinition name) def)
@@ -170,7 +169,7 @@ the file system."
   ;; also checks package locks. By doing this here we let (SETF
   ;; FDEFINITION) do the load-time package lock checking before
   ;; we frob any existing inline expansions.
-  (sb!c::%set-inline-expansion name nil inline-lambda)
+  (sb!c::%set-inline-expansion name nil inline-lambda dxable-args)
   (sb!c::note-name-defined name :function)
   name)
 
@@ -241,3 +240,9 @@ version 1[.0.0...] or greater."
                or later is required)."
               (lisp-implementation-version)
               subversions))))
+
+(!defvar sb!pcl::*!docstrings* nil)
+(defun (setf documentation) (string name doc-type)
+  (declare (type (or null string) string))
+  (push (list string name doc-type) sb!pcl::*!docstrings*)
+  string)
