@@ -137,29 +137,21 @@
       ;; translating logical pathnames to a filesystem without
       ;; versions (like Unix and Win32).
       (when name
-        (when (and (null type)
-                   (typep name 'string)
-                   (> (length name) 0)
-                   (position #\. name :start 1))
-          (no-namestring-error
-           pathname
-           "there are too many dots in the ~S component ~S." :name name))
         (when (and (typep name 'string)
                    (string= name ""))
           (no-namestring-error
            pathname "the ~S component ~S is of length 0" :name name))
-        (fragments (unparse-physical-piece name escape-char)))
+        (fragments (unparse-physical-piece
+                    name escape-char
+                    :escape-dot (when (null type) :unless-at-start))))
       (when (pathname-component-present-p type)
         (unless name
           (no-namestring-error
            pathname
            "there is a ~S component but no ~S component" :type :name))
-        (when (typep type 'simple-string)
-          (when (position #\. type)
-            (no-namestring-error
-             pathname "the ~S component contains a ~S" :type #\.)))
         (fragments ".")
-        (fragments (unparse-physical-piece type escape-char)))
+        (fragments (unparse-physical-piece
+                    type escape-char :escape-dot t)))
       (apply #'concatenate 'simple-string (fragments)))))
 
 (defun unparse-native-physical-file (pathname)
@@ -184,6 +176,54 @@
           "there is a ~S component but no ~S component" :type :name)))
       (apply #'concatenate 'simple-string (fragments)))))
 
+(defun unparse-physical-enough (pathname defaults escape-char)
+  (declare (type pathname pathname defaults))
+  (flet ((lose ()
+           (error "~S cannot be represented relative to ~S."
+                  pathname defaults)))
+    (collect ((strings))
+      (let* ((pathname-directory (%pathname-directory pathname))
+             (defaults-directory (%pathname-directory defaults))
+             (prefix-len (length defaults-directory))
+             (result-directory
+               (cond ((null pathname-directory) '(:relative))
+                     ((eq (car pathname-directory) :relative)
+                      pathname-directory)
+                     ((and (> prefix-len 0)
+                           (>= (length pathname-directory) prefix-len)
+                           (compare-component (subseq pathname-directory
+                                                      0 prefix-len)
+                                              defaults-directory))
+                      ;; Pathname starts with a prefix of default. So
+                      ;; just use a relative directory from then on out.
+                      (cons :relative (nthcdr prefix-len pathname-directory)))
+                     ((eq (car pathname-directory) :absolute)
+                      ;; We are an absolute pathname, so we can just use it.
+                      pathname-directory)
+                     (t
+                      (bug "Bad fallthrough in ~S" 'unparse-physical-enough)))))
+        (strings (unparse-physical-directory-list result-directory escape-char)))
+      (let* ((pathname-type (%pathname-type pathname))
+             (type-needed (pathname-component-present-p pathname-type))
+             (pathname-name (%pathname-name pathname))
+             (name-needed (or type-needed
+                              (and pathname-name
+                                   (not (compare-component pathname-name
+                                                           (%pathname-name
+                                                            defaults)))))))
+        (when name-needed
+          (unless pathname-name (lose))
+          (strings (unparse-physical-piece
+                    pathname-name escape-char
+                    :escape-dot (when (not pathname-type) :unless-at-start))))
+        (when type-needed
+          (unless (pathname-component-present-p pathname-type)
+            (lose))
+          (strings ".")
+          (strings (unparse-physical-piece pathname-type
+                                           escape-char :escape-dot t))))
+      (apply #'concatenate 'simple-string (strings)))))
+
 
 ;;; To be initialized in unix/win32-pathname.lisp
 (define-load-time-global *physical-host* nil)
