@@ -98,7 +98,7 @@ means to wait indefinitely.")
 (declaim (inline split-ratio-for-sleep))
 (defun split-ratio-for-sleep (seconds)
   (declare (ratio seconds)
-           (muffle-conditions t))
+           (muffle-conditions compiler-note))
   (multiple-value-bind (quot rem) (truncate (numerator seconds)
                                             (denominator seconds))
     (values quot
@@ -109,7 +109,7 @@ means to wait indefinitely.")
                      '(* 10 (truncate 100000000 (denominator seconds))))))))
 
 (defun split-seconds-for-sleep (seconds)
-  (declare (muffle-conditions t))
+  (declare (muffle-conditions compiler-note))
   ;; KLUDGE: This whole thing to avoid consing floats
   (flet ((split-float ()
            (let ((whole-seconds (truly-the fixnum (%unary-truncate seconds))))
@@ -541,6 +541,35 @@ the user for input.")
 that provides the REPL for the system. Assumes that *STANDARD-INPUT* and
 *STANDARD-OUTPUT* are set up.")
 
+;;; toplevel helper
+(defmacro with-rebound-io-syntax (&body body)
+  `(%with-rebound-io-syntax (lambda () ,@body)))
+
+(defun %with-rebound-io-syntax (function)
+  (declare (type function function))
+  (declare (dynamic-extent function))
+  (let ((*package* *package*)
+        (*print-array* *print-array*)
+        (*print-base* *print-base*)
+        (*print-case* *print-case*)
+        (*print-circle* *print-circle*)
+        (*print-escape* *print-escape*)
+        (*print-gensym* *print-gensym*)
+        (*print-length* *print-length*)
+        (*print-level* *print-level*)
+        (*print-lines* *print-lines*)
+        (*print-miser-width* *print-miser-width*)
+        (*print-pretty* *print-pretty*)
+        (*print-radix* *print-radix*)
+        (*print-readably* *print-readably*)
+        (*print-right-margin* *print-right-margin*)
+        (*read-base* *read-base*)
+        (*read-default-float-format* *read-default-float-format*)
+        (*read-eval* *read-eval*)
+        (*read-suppress* *read-suppress*)
+        (*readtable* *readtable*))
+    (funcall function)))
+
 ;;; read-eval-print loop for the default system toplevel
 (defun toplevel-repl (noprint)
   (/show0 "entering TOPLEVEL-REPL")
