@@ -21,7 +21,6 @@
   (import '(sb!vm::nil-value sb!vm::registers sb!vm::null-tn sb!vm::null-offset
             sb!vm::pc-tn sb!vm::pc-offset sb!vm::code-offset)))
 
-(defconstant +disassem-inst-alignment-bytes+ 4)
 
 
 (defconstant-eqx +conditions+
@@ -1159,7 +1158,7 @@
         ;; And finally we use the header value (a count in words),
         ;; plus the fact that the top two bits of the widetag are
         ;; clear (SIMPLE-FUN-WIDETAG is #x2A and
-        ;; RETURN-PC-HEADER-WIDETAG is #x36) to compute the boxed
+        ;; RETURN-PC-WIDETAG is #x36) to compute the boxed
         ;; address of the code component.
         (inst sub code lip (lsr temp (- 8 word-shift))))))))
 
@@ -1218,14 +1217,16 @@
                   (load-chunk segment delta
                               dest pc-tn absolute-delta))))
 
-            (two-instruction-maybe-shrink (segment posn magic-value)
+            (two-instruction-maybe-shrink (segment chooser posn magic-value)
+              (declare (ignore chooser))
               (let ((delta (compute-delta posn magic-value)))
                 (when (<= (integer-length delta) 8)
                   (emit-back-patch segment 4
                                    #'one-instruction-emitter)
                   t)))
 
-            (three-instruction-maybe-shrink (segment posn magic-value)
+            (three-instruction-maybe-shrink (segment chooser posn magic-value)
+              (declare (ignore chooser))
               (let ((delta (compute-delta posn magic-value)))
                 (when (<= (integer-length delta) 16)
                   (emit-chooser segment 8 2
@@ -1280,7 +1281,8 @@
                   (assemble (segment vop)
                     (inst ldr condition dest (@ pc-tn delta)))))
 
-              (two-instruction-maybe-shrink (segment posn magic-value)
+              (two-instruction-maybe-shrink (segment chooser posn magic-value)
+                (declare (ignore chooser))
                 (let ((delta (compute-delta posn magic-value)))
                   (when (<= (integer-length delta) 12)
                     (emit-back-patch segment 4
