@@ -10,14 +10,10 @@
 ;;;; files for more information.
 (in-package "SB!VM")
 
-(defparameter *immediate-types*
+(defconstant-eqx +immediate-types+
   (list unbound-marker-widetag character-widetag
-        #!+64-bit single-float-widetag))
-
-(defparameter *fun-header-widetags*
-  (list funcallable-instance-header-widetag
-        simple-fun-header-widetag
-        closure-header-widetag))
+        #!+64-bit single-float-widetag)
+  #'equal)
 
 ;; Given a list of widetags in HEADERS, compress into a minimal list of ranges
 ;; and/or singletons that should be tested.
@@ -25,7 +21,7 @@
 ;;        in an ad-hoc way by each backend. The range convention should be
 ;;        changed to indicate explicitly when either limit needn't be checked.
 ;;        (Use NIL or * as a bound perhaps)
-(defun canonicalize-headers (headers)
+(defun canonicalize-widetags (headers)
   (collect ((results))
     (let ((start nil)
           (prev nil)
@@ -54,19 +50,29 @@
 ;; or return the unmodified ranges and NIL.
 ;; This could be generalized: three ranges that collapse to one with at most
 ;; two exceptions, or three collapsing to two with one exception, etc.
-(defun canonicalize-headers-and-exceptions (widetags)
-  (let ((ranges (canonicalize-headers widetags)))
-    (if (and (cdr ranges) (endp (cddr ranges))
-             (listp (car ranges)) (listp (cadr ranges)) ; 2 ranges
-             (let ((end-range-1 (cdar ranges))
-                   (start-range-2 (caadr ranges)))
-               (= start-range-2 (+ end-range-1 8))))
-        ;; Return ((start-range-1 . end-range-2))
-        (values (list (cons (caar ranges) (cdadr ranges)))
-                (list (+ (cdar ranges) 4))) ; the excluded value
-        (values ranges nil))))
+(defun canonicalize-widetags+exceptions (widetags)
+  (let ((ranges (canonicalize-widetags widetags)))
+    (flet ((begin (x) (if (listp x) (car x) x))
+           (end (x) (if (listp x) (cdr x) x)))
+      (when (and (cdr ranges) (endp (cddr ranges))) ; 2 ranges
+        (let* ((range-1 (first ranges))
+               (range-2 (second ranges))
+               (begin-1 (begin range-1))
+               (end-1 (end range-1))
+               (begin-2 (begin range-2))
+               (end-2 (end range-2))
+               (delta (- other-immediate-1-lowtag other-immediate-0-lowtag)))
+          (when (and (= (+ end-1 (* 2 delta)) begin-2)
+                     ;; Don't return {X} - {Y} if {X} spans only 3 widetags,
+                     ;; because clearly we can just test the 2 members of X.
+                     ;; fencepost: 3 delta is 4 widetags.
+                     (>= (- end-2 begin-1) (* 3 delta)))
+            (return-from canonicalize-widetags+exceptions
+              (values `((,begin-1 . ,end-2))
+                      `(,(+ end-1 delta)))))))) ; the excluded value
+    (values ranges nil)))
 
-(defmacro test-type (value target not-p
+(defmacro test-type (value temp target not-p
                      (&rest type-codes)
                      &rest other-args
                      &key &allow-other-keys)
@@ -78,7 +84,7 @@
                        t))
          (lowtags (remove lowtag-limit type-codes :test #'<))
          (extended (remove lowtag-limit type-codes :test #'>))
-         (immediates (intersection extended *immediate-types* :test #'eql))
+         (immediates (intersection extended +immediate-types+ :test #'eql))
          ;; To collapse the range of widetags comprising real numbers on 64-bit
          ;; machines, consider SHORT-FLOAT-WIDETAG both a header and immediate.
          ;; No OTHER-POINTER-LOWTAG object can ever have that header tag.
@@ -92,14 +98,14 @@
                    (if (and (= n-word-bits 64)
                             (member (- single-float-widetag 4) extended)
                             (member (+ single-float-widetag 4) extended))
-                       (remove single-float-widetag *immediate-types*)
-                       *immediate-types*)
+                       (remove single-float-widetag +immediate-types+)
+                       +immediate-types+)
                    :test #'eql))
-         (function-p (if (intersection headers *fun-header-widetags*)
-                         (if (subsetp headers *fun-header-widetags*)
+         (function-p (if (intersection headers +function-widetags+)
+                         (if (subsetp headers +function-widetags+)
                              t
                              (error "can't test for mix of function subtypes ~
-                                     and normal header types"))
+                                     and other header types"))
                          nil)))
     (unless type-codes
       (error "At least one type must be supplied for TEST-TYPE."))
@@ -113,31 +119,31 @@
          (error "can't mix fixnum testing with function subtype testing"))
        (cond
          ((and (= n-word-bits 64) immediates headers)
-          `(%test-fixnum-immediate-and-headers ,value ,target ,not-p
+          `(%test-fixnum-immediate-and-headers ,value ,temp ,target ,not-p
                                                ,(car immediates)
-                                               ',(canonicalize-headers
+                                               ',(canonicalize-widetags
                                                   headers)
                                                ,@other-args))
          (immediates
           (if (= n-word-bits 64)
-              `(%test-fixnum-and-immediate ,value ,target ,not-p
+              `(%test-fixnum-and-immediate ,value ,temp ,target ,not-p
                                            ,(car immediates)
                                            ,@other-args)
               (error "can't mix fixnum testing with other immediates")))
          (headers
-          `(%test-fixnum-and-headers ,value ,target ,not-p
-                                     ',(canonicalize-headers headers)
+          `(%test-fixnum-and-headers ,value ,temp ,target ,not-p
+                                     ',(canonicalize-widetags headers)
                                      ,@other-args))
          (t
-          `(%test-fixnum ,value ,target ,not-p
+          `(%test-fixnum ,value ,temp ,target ,not-p
                          ,@other-args))))
       (immediates
        (cond
          (headers
           (if (= n-word-bits 64)
-              `(%test-immediate-and-headers ,value ,target ,not-p
+              `(%test-immediate-and-headers ,value ,temp ,target ,not-p
                                             ,(car immediates)
-                                            ',(canonicalize-headers headers)
+                                            ',(canonicalize-widetags headers)
                                             ,@other-args)
               (error "can't mix testing of immediates with testing of headers")))
          (lowtags
@@ -145,17 +151,17 @@
          ((cdr immediates)
           (error "can't test multiple immediates at the same time"))
          (t
-          `(%test-immediate ,value ,target ,not-p ,(car immediates)
+          `(%test-immediate ,value ,temp ,target ,not-p ,(car immediates)
                             ,@other-args))))
       (lowtags
        (when (cdr lowtags)
          (error "can't test multiple lowtags at the same time"))
        (when headers
          (error "can't test non-fixnum lowtags and headers at the same time"))
-       `(%test-lowtag ,value ,target ,not-p ,(car lowtags) ,@other-args))
+       `(%test-lowtag ,value ,temp ,target ,not-p ,(car lowtags) ,@other-args))
       (headers
-       `(%test-headers ,value ,target ,not-p ,function-p
-         ',(canonicalize-headers headers)
+       `(%test-headers ,value ,temp ,target ,not-p ,function-p
+         ',(canonicalize-widetags headers)
          ,@other-args))
       (t
        (error "nothing to test?")))))
