@@ -100,11 +100,6 @@
 ;; Used during compilation to keep track of with source paths have been
 ;; instrumented in which blocks.
 (defun code-coverage-blocks (x) (cdr x))
-;; Stores the code coverage instrumentation results. Keys are namestrings, the
-;; value is a list of (CONS PATH STATE), where STATE is +CODE-COVERAGE-UNMARKED+
-;; for a path that has not been visited, and T for one that has.
-(defvar *code-coverage-info* (make-hash-table :test 'equal))
-
 
 ;;;; WITH-COMPILATION-UNIT and WITH-COMPILATION-VALUES
 
@@ -191,6 +186,7 @@ Examples:
 
 (defun %with-compilation-unit (fn &key override policy source-plist source-namestring)
   (declare (type function fn))
+  (declare (dynamic-extent fn))
   (flet ((with-it ()
            (let ((succeeded-p nil)
                  (*source-plist* (append source-plist *source-plist*))
@@ -444,35 +440,36 @@ necessary, since type inference may take arbitrarily long to converge.")
         (cleared-reanalyze nil)
         (fastp nil))
     (loop
-      (when (component-reanalyze component)
-        (setq count 0)
-        (setq cleared-reanalyze t)
-        (setf (component-reanalyze component) nil))
-      (setf (component-reoptimize component) nil)
-      (ir1-optimize component fastp)
-      (cond ((component-reoptimize component)
-             (incf count)
-             (when (and (>= count *max-optimize-iterations*)
-                        (not (component-reanalyze component))
-                        (eq (component-reoptimize component) :maybe))
-               (maybe-mumble "*")
-               (cond ((retry-delayed-ir1-transforms :optimize)
-                      (maybe-mumble "+")
-                      (setq count 0))
-                     (t
-                      (event ir1-optimize-maxed-out)
-                      (setf (component-reoptimize component) nil)
-                      (do-blocks (block component)
-                        (setf (block-reoptimize block) nil))
-                      (return)))))
-            ((retry-delayed-ir1-transforms :optimize)
-             (setf count 0)
-             (maybe-mumble "+"))
-            (t
-             (maybe-mumble " ")
-             (return)))
-      (setq fastp (>= count *max-optimize-iterations*))
-      (maybe-mumble (if fastp "-" ".")))
+     (when (component-reanalyze component)
+       (setf count 0
+             fastp nil
+             cleared-reanalyze t
+             (component-reanalyze component) nil))
+     (setf (component-reoptimize component) nil)
+     (ir1-optimize component fastp)
+     (cond ((component-reoptimize component)
+            (incf count)
+            (when (and (>= count *max-optimize-iterations*)
+                       (not (component-reanalyze component))
+                       (eq (component-reoptimize component) :maybe))
+              (maybe-mumble "*")
+              (cond ((retry-delayed-ir1-transforms :optimize)
+                     (maybe-mumble "+")
+                     (setq count 0))
+                    (t
+                     (event ir1-optimize-maxed-out)
+                     (setf (component-reoptimize component) nil)
+                     (do-blocks (block component)
+                       (setf (block-reoptimize block) nil))
+                     (return)))))
+           ((retry-delayed-ir1-transforms :optimize)
+            (setf count 0)
+            (maybe-mumble "+"))
+           (t
+            (maybe-mumble " ")
+            (return)))
+     (setq fastp (>= count *max-optimize-iterations*))
+     (maybe-mumble (if fastp "-" ".")))
     (when cleared-reanalyze
       (setf (component-reanalyze component) t)))
   (values))
@@ -527,6 +524,7 @@ necessary, since type inference may take arbitrarily long to converge.")
         (maybe-mumble "constraint ")
         (constraint-propagate component))
       (when (retry-delayed-ir1-transforms :constraint)
+        (setf loop-count 0) ;; otherwise nothing may get retried
         (maybe-mumble "Rtran "))
       (flet ((want-reoptimization-p ()
                (or (component-reoptimize component)
@@ -574,10 +572,7 @@ necessary, since type inference may take arbitrarily long to converge.")
         (eq *compile-to-memory-space* :immobile))))
 
 (defun %compile-component (component)
-  (let ((*code-segment* nil)
-        (*elsewhere* nil)
-        (*elsewhere-label* nil)
-        #!+inline-constants (*unboxed-constants* nil))
+  (let ((*adjustable-vectors* nil)) ; Needed both by codegen and fasl writer
     (maybe-mumble "GTN ")
     (gtn-analyze component)
     (maybe-mumble "LTN ")
@@ -603,7 +598,6 @@ necessary, since type inference may take arbitrarily long to converge.")
     (unwind-protect
         (progn
           (maybe-mumble "IR2tran ")
-          (init-assembler)
           (entry-analyze component)
           (ir2-convert component)
 
@@ -648,26 +642,38 @@ necessary, since type inference may take arbitrarily long to converge.")
 
           (maybe-mumble "code ")
 
-          (multiple-value-bind (code-length fixup-notes)
+          (multiple-value-bind (segment text-length fun-table
+                                elsewhere-label fixup-notes)
               (let (#!+immobile-code
                     (*code-is-immobile* (code-immobile-p component)))
                 (generate-code component))
 
-            #-sb-xc-host
-            (when *compiler-trace-output*
-              (format *compiler-trace-output*
-                      "~|~%disassembly of code for ~S~2%" component)
-              (sb!disassem:disassemble-assem-segment *code-segment*
-                                                     *compiler-trace-output*))
+            (let ((bytes (sb!assem:segment-contents-as-vector segment))
+                  (object *compile-object*)
+                  (*elsewhere-label* elsewhere-label)) ; KLUDGE
+
+              (when *compiler-trace-output*
+                (let ((ranges
+                       (maplist (lambda (list)
+                                  (cons (+ (car list)
+                                           (ash sb!vm:simple-fun-code-offset
+                                                sb!vm:word-shift))
+                                        (or (cadr list) text-length)))
+                                fun-table)))
+                  (declare (ignorable ranges))
+                  (format *compiler-trace-output*
+                          "~|~%disassembly of code for ~S~2%" component)
+                  #-sb-xc-host
+                  (sb!disassem:disassemble-assem-segment
+                   bytes ranges *compiler-trace-output*)))
 
-            (let ((object *compile-object*))
               (funcall (etypecase object
                         (fasl-output (maybe-mumble "fasl") #'fasl-dump-component)
                         #-sb-xc-host ; no compiling to core
                         (core-object (maybe-mumble "core") #'make-core-component)
                         (null (lambda (&rest dummies)
                                 (declare (ignore dummies)))))
-                       component *code-segment* code-length fixup-notes
+                       component segment (length bytes) fixup-notes
                        object))))))
 
   ;; We're done, so don't bother keeping anything around.
@@ -902,6 +908,8 @@ necessary, since type inference may take arbitrarily long to converge.")
 ;; a subtype of not-so-aptly-named INPUT-ERROR-IN-COMPILE-FILE.
 (defun %do-forms-from-info (function info condition-name)
   (declare (function function))
+  ; FIXME: why "could not stack-allocate" in a few call sites?
+  ; (declare (dynamic-extent function))
   (let* ((file-info (source-info-file-info info))
          (stream (get-source-stream info))
          (pos (file-position stream))
@@ -1241,50 +1249,35 @@ necessary, since type inference may take arbitrarily long to converge.")
           (multiple-value-bind (result found-p)
               (gethash (leaf-info fun) entry-table)
             (aver found-p)
-            (prog1
-                result
-              ;; KLUDGE: This code duplicates some other code in this
-              ;; file. In the great reorganzation, the flow of program
-              ;; logic changed from the original CMUCL model, and that
-              ;; path (as of sbcl-0.7.5 in SUB-COMPILE-FILE) was no
-              ;; longer followed for CORE-OBJECTS, leading to BUG
-              ;; 156. This place is transparently not the right one for
-              ;; this code, but I don't have a clear enough overview of
-              ;; the compiler to know how to rearrange it all so that
-              ;; this operation fits in nicely, and it was blocking
-              ;; reimplementation of (DECLAIM (INLINE FOO)) (MACROLET
-              ;; ((..)) (DEFUN FOO ...))
-              ;;
-              ;; FIXME: This KLUDGE doesn't solve all the problem in an
-              ;; ideal way, as (1) definitions typed in at the REPL
-              ;; without an INLINE declaration will give a NULL
-              ;; FUNCTION-LAMBDA-EXPRESSION (allowable, but not ideal)
-              ;; and (2) INLINE declarations will yield a
-              ;; FUNCTION-LAMBDA-EXPRESSION headed by
-              ;; SB-C:LAMBDA-WITH-LEXENV, even for null LEXENV.  -- CSR,
-              ;; 2002-07-02
-              ;;
-              ;; (2) is probably fairly easy to fix -- it is, after all,
-              ;; a matter of list manipulation (or possibly of teaching
-              ;; CL:FUNCTION about SB-C:LAMBDA-WITH-LEXENV).  (1) is
-              ;; significantly harder, as the association between
-              ;; function object and source is a tricky one.
-              ;;
-              ;; FUNCTION-LAMBDA-EXPRESSION "works" (i.e. returns a
-              ;; non-NULL list) when the function in question has been
-              ;; compiled by (COMPILE <x> '(LAMBDA ...)); it does not
-              ;; work when it has been compiled as part of the top-level
-              ;; EVAL strategy of compiling everything inside (LAMBDA ()
-              ;; ...).  -- CSR, 2002-11-02
-              (when (core-object-p object)
-                #+sb-xc-host (error "Can't compile to core")
-                #-sb-xc-host
-                (fix-core-source-info *source-info* object
-                                      (and (policy (lambda-bind fun)
-                                               (> eval-store-source-form 0))
-                                           result)))
 
-              (mapc #'clear-ir1-info components-from-dfo))))))))
+            (when (core-object-p object)
+              #+sb-xc-host (error "Can't compile to core")
+              #-sb-xc-host
+              (let ((store-source
+                     (policy (lambda-bind fun)
+                             (> eval-store-source-form 0))))
+                (fix-core-source-info *source-info* object
+                                      (and store-source result))
+                ;; FIXME: here on down the logic probably belongs in
+                ;; MAKE-CORE-COMPONENT, but to do that we'd want to pass in
+                ;; the EVAL-STORE-SOURCE policy
+                (when store-source
+                  ;; Store each simple-fun's lambda expression
+                  (dohash ((entry simple-fun) entry-table)
+                    (let ((info (entry-info-info entry)))
+                      (sb!impl::set-simple-fun-info
+                         simple-fun
+                         (entry-info-lexpr entry)
+                         ;; SET-SIMPLE-FUN-INFO expects all 3 thingies that we might
+                         ;; save (any can be NIL) so extract the existing docstring
+                         (cond ((listp info) (car info))
+                               ((stringp info) info))
+                         ;; ... and the existing xrefs
+                         (cond ((listp info) (cdr info))
+                               ((simple-vector-p info) info))))))))
+
+            (mapc #'clear-ir1-info components-from-dfo)
+            result))))))
 
 (defun note-top-level-form (form &optional finalp)
   (when *compile-print*
@@ -1351,8 +1344,8 @@ necessary, since type inference may take arbitrarily long to converge.")
               (convert-and-maybe-compile
                (make-compiler-error-form condition form)
                path)
-              (throw 'process-toplevel-form-error-abort nil))))
-
+              (throw 'process-toplevel-form-error-abort nil)))
+           (*top-level-form-p* t))
       (flet ((default-processor (form)
                (let ((*top-level-form-noted* (note-top-level-form form)))
                  ;; When we're cross-compiling, consider: what should we
@@ -1414,7 +1407,8 @@ necessary, since type inference may take arbitrarily long to converge.")
                    (cond ((eq expanded form)
                           (when compile-time-too
                             (eval-compile-toplevel (list form) path))
-                          (convert-and-maybe-compile form path nil))
+                          (let (*top-level-form-p*)
+                            (convert-and-maybe-compile form path nil)))
                          (t
                           (process-toplevel-form expanded
                                                  path
@@ -1709,7 +1703,7 @@ necessary, since type inference may take arbitrarily long to converge.")
         ;; compiler to compile code for itself which isn't sanitized,
         ;; *or* code for another image which is sanitized.
         ;; And we can also cross-compile assuming msan.
-        (*msan-unpoison* (member :msan (sb!fasl::fasl-target-features)))
+        (*msan-unpoison* (member :msan sb!xc:*features*))
         (*handled-conditions* *handled-conditions*)
         (*disabled-package-locks* *disabled-package-locks*)
         (*lexenv* (make-null-lexenv))
@@ -1737,7 +1731,7 @@ necessary, since type inference may take arbitrarily long to converge.")
                   ;; Dump the code coverage records into the fasl.
                    (with-source-paths
                     (fopcompile `(record-code-coverage
-                                  ',(namestring *compile-file-pathname*)
+                                  ',(namestring sb!xc:*compile-file-pathname*)
                                   ',(let (list)
                                       (maphash (lambda (k v)
                                                  (declare (ignore k))
