@@ -40,6 +40,7 @@
     (inst shl header n-widetag-bits)
     (inst or  header type)
     (inst shr header n-fixnum-tag-bits)
+    (instrument-alloc bytes node)
     (pseudo-atomic
      (allocation result bytes node nil other-pointer-lowtag)
      (storew header result 0 other-pointer-lowtag))))
@@ -54,12 +55,11 @@
   (:generator 12
     (let* ((header-size (+ rank
                            (1- array-dimensions-offset)))
-           (bytes (logandc2 (+ (* (1+ header-size) n-word-bytes)
-                               lowtag-mask)
-                            lowtag-mask))
+           (bytes (* (align-up (1+ header-size) 2) n-word-bytes))
            (header (logior (ash header-size
                                 n-widetag-bits)
                            type)))
+     (instrument-alloc bytes node)
      (pseudo-atomic
       (allocation result bytes node nil other-pointer-lowtag)
       (storew header result 0 other-pointer-lowtag)))))
@@ -104,55 +104,33 @@
                      :index res
                      :base (if (= scale 2) res nil)
                      :disp (- (* scale (1- array-dimensions-offset))))))))
+
+(define-vop (array-rank=)
+  (:translate %array-rank=)
+  (:policy :fast-safe)
+  (:args (array :scs (descriptor-reg)))
+  (:info rank)
+  (:arg-types * (:constant t))
+  (:conditional :e)
+  (:generator 2
+    (inst cmp (make-ea :dword :base array :disp (1+ (- other-pointer-lowtag)))
+          (+ rank
+             (1- array-dimensions-offset)))))
 
 ;;;; bounds checking routine
-
-;;; Note that the immediate SC for the index argument is disabled
-;;; because it is not possible to generate a valid error code SC for
-;;; an immediate value.
-;;;
-;;; FIXME: As per the KLUDGE note explaining the :IGNORE-FAILURE-P
-;;; flag in build-order.lisp-expr, compiling this file causes warnings
-;;;    Argument FOO to VOP CHECK-BOUND has SC restriction
-;;;    DESCRIPTOR-REG which is not allowed by the operand type:
-;;;      (:OR POSITIVE-FIXNUM)
-;;; CSR's message "format ~/ /" on sbcl-devel 2002-03-12 contained
-;;; a possible patch, described as
-;;;   Another patch is included more for information than anything --
-;;;   removing the descriptor-reg SCs from the CHECK-BOUND vop in
-;;;   x86/array.lisp seems to allow that file to compile without error[*],
-;;;   and build; I haven't tested rebuilding capability, but I'd be
-;;;   surprised if there were a problem.  I'm not certain that this is the
-;;;   correct fix, though, as the restrictions on the arguments to the VOP
-;;;   aren't the same as in the sparc and alpha ports, where, incidentally,
-;;;   the corresponding file builds without error currently.
-;;; Since neither of us (CSR or WHN) was quite sure that this is the
-;;; right thing, I've just recorded the patch here in hopes it might
-;;; help when someone attacks this problem again:
-;;;   diff -u -r1.7 array.lisp
-;;;   --- src/compiler/x86/array.lisp 11 Oct 2001 14:05:26 -0000      1.7
-;;;   +++ src/compiler/x86/array.lisp 12 Mar 2002 12:23:37 -0000
-;;;   @@ -76,10 +76,10 @@
-;;;      (:translate %check-bound)
-;;;      (:policy :fast-safe)
-;;;      (:args (array :scs (descriptor-reg))
-;;;   -        (bound :scs (any-reg descriptor-reg))
-;;;   -        (index :scs (any-reg descriptor-reg #+nil immediate) :target result))
-;;;   +        (bound :scs (any-reg))
-;;;   +        (index :scs (any-reg #+nil immediate) :target result))
-;;;      (:arg-types * positive-fixnum tagged-num)
-;;;   -  (:results (result :scs (any-reg descriptor-reg)))
-;;;   +  (:results (result :scs (any-reg)))
-;;;      (:result-types positive-fixnum)
-;;;      (:vop-var vop)
-;;;      (:save-p :compute-only)
 (define-vop (check-bound)
   (:translate %check-bound)
   (:policy :fast-safe)
-  (:args (array :scs (descriptor-reg))
-         (bound :scs (any-reg descriptor-reg))
-         (index :scs (any-reg descriptor-reg)))
-  ;(:arg-types * positive-fixnum *)
+  (:args (array :scs (descriptor-reg constant))
+         (bound :scs (any-reg descriptor-reg)
+                :load-if (not (and (sc-is bound immediate)
+                                   (typep (tn-value bound)
+                                          'sc-offset)
+                                   (not (sc-is index immediate)))))
+         (index :scs (any-reg descriptor-reg)
+                :load-if (not (and (sc-is index immediate)
+                                   (typep (tn-value index)
+                                          'sc-offset)))))
   (:variant-vars %test-fixnum)
   (:variant t)
   (:vop-var vop)
@@ -160,20 +138,32 @@
   (:generator 5
     (let ((error (generate-error-code vop 'invalid-array-index-error
                                       array bound index))
+          (bound (if (sc-is bound immediate)
+                     (fixnumize (tn-value bound))
+                     bound))
           (index (if (sc-is index immediate)
                      (fixnumize (tn-value index))
                      index)))
       (when (and %test-fixnum (not (integerp index)))
-        (%test-fixnum index error t))
-      (inst cmp bound index)
-      ;; We use below-or-equal even though it's an unsigned test,
-      ;; because negative indexes appear as large unsigned numbers.
-      ;; Therefore, we get the <0 and >=bound test all rolled into one.
-      (inst jmp :be error))))
+        (%test-fixnum index nil error t))
+      (cond ((integerp bound)
+             (inst cmp index bound)
+             (inst jmp :nb error))
+            (t
+             (inst cmp bound index)
+             (inst jmp :be error))))))
 (define-vop (check-bound/fast check-bound)
   (:policy :fast)
   (:variant nil)
   (:variant-cost 4))
+
+(define-vop (check-bound/fixnum check-bound)
+  (:args (array)
+         (bound)
+         (index :scs (any-reg)))
+  (:arg-types * * tagged-num)
+  (:variant nil)
+  (:variant-cost 4))
 
 ;;;; accessors/setters
 
@@ -189,9 +179,8 @@
                 (define-full-setter+offset
                   ,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" type)
                   ,type vector-data-offset other-pointer-lowtag ,scs
-                  ,element-type data-vector-set-with-offset)))
-           )
-  (def-full-data-vector-frobs simple-vector * descriptor-reg any-reg)
+                  ,element-type data-vector-set-with-offset))))
+  (def-full-data-vector-frobs simple-vector * descriptor-reg any-reg immediate)
   (def-full-data-vector-frobs simple-array-unsigned-byte-64 unsigned-num
     unsigned-reg)
   (def-full-data-vector-frobs simple-array-fixnum tagged-num any-reg)
@@ -859,3 +848,26 @@
                                  other-pointer-lowtag))
           diff :lock)
     (move result diff)))
+
+(defknown layout-inherits-ref-eq (simple-vector index t) boolean (flushable))
+(define-vop (layout-inherits-ref-eq)
+  (:translate layout-inherits-ref-eq)
+  (:policy :fast-safe)
+  (:conditional :e)
+  (:args (vector :scs (descriptor-reg))
+         (index :scs (any-reg descriptor-reg immediate))
+         (thing :scs (descriptor-reg immediate)))
+  (:generator 1
+    (inst cmp
+          (if (sc-is index immediate)
+              (make-ea :dword :base vector
+                       :disp (+ (- other-pointer-lowtag)
+                                (ash (+ vector-data-offset (tn-value index))
+                                     word-shift)))
+              (make-ea :dword :base vector
+                       :disp (+ (- other-pointer-lowtag)
+                                (ash vector-data-offset word-shift))
+                       :index index :scale (ash 1 (- word-shift n-fixnum-tag-bits))))
+          (if (sc-is thing immediate)
+              (make-fixup (tn-value thing) :layout)
+              thing))))
