@@ -27,11 +27,36 @@
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg immediate)))
   (:info name offset lowtag)
- ;(:ignore name)
   (:results)
+  (:vop-var vop)
   (:generator 1
-    (progn name) ; ignore it
-    (gen-cell-set (make-ea-for-object-slot object offset lowtag) value nil)))
+    (cond ((emit-gc-barrier-store-p name)
+           ;; Don't assume that the immediate is acceptable to 'push'
+           ;; (only MOV can take a 64-bit immediate)
+           (inst mov temp-reg-tn (encode-value-if-immediate value))
+           (inst push temp-reg-tn)
+           ;; Push the slot index into code, then code itself
+           (cond ((= lowtag fun-pointer-lowtag)
+                  ;; compute code address similarly to CODE-FROM-FUNCTION vop
+                  (inst mov (reg-in-size temp-reg-tn :dword)
+                        (make-ea-for-object-slot-half object 0 fun-pointer-lowtag))
+                  (inst shr (reg-in-size temp-reg-tn :dword) n-widetag-bits)
+                  ;; now temp-reg-tn holds the difference in words from code to fun.
+                  (inst push temp-reg-tn)
+                  (inst add (make-ea :qword :base rsp-tn) offset) ; for particular slot
+                  ;; finish computing the code address
+                  (inst neg temp-reg-tn)
+                  (inst lea temp-reg-tn
+                        (make-ea :qword :base object :index temp-reg-tn :scale n-word-bytes
+                                 :disp (- other-pointer-lowtag fun-pointer-lowtag)))
+                  (inst push temp-reg-tn))
+                 (t ; is code already
+                  (inst push offset)
+                  (inst push object)))
+           (invoke-asm-routine 'call 'code-header-set vop))
+          (t
+           (gen-cell-set (make-ea-for-object-slot object offset lowtag)
+                         value nil)))))
 
 ;; INIT-SLOT has to know about the :COMPACT-INSTANCE-HEADER feature.
 (define-vop (init-slot set-slot)
@@ -397,7 +422,7 @@
   #!+immobile-code (:temporary (:sc unsigned-reg) temp)
   (:generator 38
     #!+immobile-code
-    (let ((tramp (make-fixup 'undefined-tramp-fdefn :assembly-routine)))
+    (let ((tramp (make-fixup 'undefined-fdefn :assembly-routine)))
      (if (sb!c::code-immobile-p vop)
          (inst lea temp (make-ea :qword :base rip-tn :disp tramp))
          (inst mov temp tramp))
@@ -454,9 +479,7 @@
   (:info symbol)
   (:generator 10
     (inst mov bsp (* binding-size n-word-bytes))
-    (inst xadd
-          (thread-tls-ea (ash thread-binding-stack-pointer-slot word-shift))
-          bsp)
+    (inst xadd (thread-slot-ea thread-binding-stack-pointer-slot) bsp)
     (let* ((tls-index (load-time-tls-offset symbol))
            (tls-cell (thread-tls-ea tls-index)))
       ;; Too bad we can't use "XCHG [thread + disp], val" to write new value
@@ -476,10 +499,10 @@
          (symbol :scs (descriptor-reg)))
   (:temporary (:sc unsigned-reg) temp bsp)
   (:generator 5
-    (load-symbol-value bsp *binding-stack-pointer*)
+    (load-binding-stack-pointer bsp)
     (loadw temp symbol symbol-value-slot other-pointer-lowtag)
     (inst add bsp (* binding-size n-word-bytes))
-    (store-symbol-value bsp *binding-stack-pointer*)
+    (store-binding-stack-pointer bsp)
     (storew temp bsp (- binding-value-slot binding-size))
     (storew symbol bsp (- binding-symbol-slot binding-size))
     (storew val symbol symbol-value-slot other-pointer-lowtag)))
@@ -509,14 +532,14 @@
 (define-vop (unbind)
   (:temporary (:sc unsigned-reg) symbol value bsp)
   (:generator 0
-    (load-symbol-value bsp *binding-stack-pointer*)
+    (load-binding-stack-pointer bsp)
     (loadw symbol bsp (- binding-symbol-slot binding-size))
     (loadw value bsp (- binding-value-slot binding-size))
     (storew value symbol symbol-value-slot other-pointer-lowtag)
     (storew 0 bsp (- binding-symbol-slot binding-size))
     (storew 0 bsp (- binding-value-slot binding-size))
     (inst sub bsp (* binding-size n-word-bytes))
-    (store-symbol-value bsp *binding-stack-pointer*)))
+    (store-binding-stack-pointer bsp)))
 
 (define-vop (unbind-to-here)
   (:args (where :scs (descriptor-reg any-reg)))
@@ -661,12 +684,17 @@
 (define-full-reffer code-header-ref * 0 other-pointer-lowtag
   (any-reg descriptor-reg) * code-header-ref)
 
-(define-full-setter code-header-set * 0 other-pointer-lowtag
+;; CODE-HEADER-SET of a constant index is never used, so we specifically
+;; ask not to get it defined. (It's not impossible to use, it's just not
+;; helpful to have - either you want to access CODE-FIXUPS or CODE-DEBUG-INFO,
+;; which do things their own way, or you want a variable index.)
+(define-full-setter (code-header-set :no-constant-variant)
+  * 0 other-pointer-lowtag
   (any-reg descriptor-reg) * code-header-set)
 
 ;;;; raw instance slot accessors
 
-(flet ((make-ea-for-raw-slot (object index)
+(flet ((instance-slot-ea (object index)
          (etypecase index
            (integer
               (make-ea :qword
@@ -691,7 +719,7 @@
               (:results (value :scs (,result-sc)))
               (:result-types ,result-type)
               (:generator 5
-                (inst ,inst value (make-ea-for-raw-slot object index))))
+                (inst ,inst value (instance-slot-ea object index))))
             (define-vop (,(symbolicate "RAW-INSTANCE-REF-C/" suffix))
               (:translate ,(symbolicate "%RAW-INSTANCE-REF/" suffix))
               (:policy :fast-safe)
@@ -705,7 +733,7 @@
               (:results (value :scs (,result-sc)))
               (:result-types ,result-type)
               (:generator 4
-                (inst ,inst/c value (make-ea-for-raw-slot object index))))
+                (inst ,inst/c value (instance-slot-ea object index))))
             (define-vop (,(symbolicate "RAW-INSTANCE-SET/" suffix))
               (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
               (:policy :fast-safe)
@@ -716,7 +744,7 @@
               (:results (result :scs (,result-sc)))
               (:result-types ,result-type)
               (:generator 5
-                (inst ,inst (make-ea-for-raw-slot object index) value)
+                (inst ,inst (instance-slot-ea object index) value)
                 (move result value)))
             (define-vop (,(symbolicate "RAW-INSTANCE-SET-C/" suffix))
               (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
@@ -731,7 +759,7 @@
               (:results (result :scs (,result-sc)))
               (:result-types ,result-type)
               (:generator 4
-                (inst ,inst/c (make-ea-for-raw-slot object index) value)
+                (inst ,inst/c (instance-slot-ea object index) value)
                 (move result value)))
             (define-vop (,(symbolicate "RAW-INSTANCE-INIT/" suffix))
               (:args (object :scs (descriptor-reg))
@@ -739,7 +767,7 @@
               (:arg-types * ,result-type)
               (:info index)
               (:generator 4
-                (inst ,inst/c (make-ea-for-raw-slot object index) value))))))
+                (inst ,inst/c (instance-slot-ea object index) value))))))
     (def word unsigned-reg unsigned-num mov)
     (def signed-word signed-reg signed-num mov)
     (def single single-reg single-float movss)
@@ -758,7 +786,7 @@
     (:results (result :scs (unsigned-reg)))
     (:result-types unsigned-num)
     (:generator 5
-      (inst xadd (make-ea-for-raw-slot object index) diff :lock)
+      (inst xadd (instance-slot-ea object index) diff :lock)
       (move result diff)))
 
   (define-vop (raw-instance-atomic-incf-c/word)
@@ -774,7 +802,7 @@
     (:results (result :scs (unsigned-reg)))
     (:result-types unsigned-num)
     (:generator 4
-      (inst xadd (make-ea-for-raw-slot object index) diff :lock)
+      (inst xadd (instance-slot-ea object index) diff :lock)
       (move result diff))))
 
 ;;;;
