@@ -105,7 +105,10 @@
           (labels ((make-tn ()
                      (make-representation-tn ptype scn))
                    (frob-tn (tn)
-                     (if (immediate-tn-p tn)
+                     ;; Careful not to load constants which require boxing
+                     ;; and may overwrite the flags.
+                     ;; Representation selection should avoid that.
+                     (if (eq (tn-kind tn) :constant)
                          tn
                          (make-tn))))
             (values vop
@@ -204,7 +207,7 @@
     (cond
       ((sc-is y immediate)
        (let* ((value (encode-value-if-immediate y))
-              (immediate (immediate32-p value)))
+              (immediate (plausible-signed-imm32-operand-p value)))
          (cond ((fixup-p value) ; immobile object
                 (inst cmp x value))
                ((and (zerop value) (sc-is x any-reg descriptor-reg))
