@@ -372,15 +372,11 @@ adjust_code_refs(struct heap_adjust* adj, struct code* code, lispobj original_va
     // Dynamic space is a always relocated before immobile space,
     // and dynamic space code does not use fixups (except on 32-bit x86).
     // So if we're here, it must be to relocate an immobile object.
-    // Therefore these CAR and CDR operations are correct,
-    // because what they point to has already been relocated.
-    lispobj fixups    = code->fixups;
-    lispobj absfixups = listp(fixups) ? CONS(fixups)->car : fixups;
-    lispobj relfixups = listp(fixups) ? CONS(fixups)->cdr : 0;
+    // If 'code->fixups' is a bignum, the pointer itself was already fixed up.
     char* instructions = (char*)((lispobj*)code + code_header_words(code->header));
     struct varint_unpacker unpacker;
 
-    varint_unpacker_init(&unpacker, absfixups);
+    varint_unpacker_init(&unpacker, code->fixups);
     int prev_loc = 0, loc;
     while (varint_unpack(&unpacker, &loc) && loc != 0) {
         // For extra compactness, each loc is relative to the prior,
@@ -396,7 +392,6 @@ adjust_code_refs(struct heap_adjust* adj, struct code* code, lispobj original_va
     sword_t displacement = (lispobj)code - original_vaddr;
     if (!displacement) // if this code didn't move, do nothing
         return;
-    varint_unpacker_init(&unpacker, relfixups);
     prev_loc = 0;
     while (varint_unpack(&unpacker, &loc) && loc != 0) {
         loc += prev_loc;
@@ -419,6 +414,37 @@ static inline void fix_fun_header_layout(lispobj* fun, struct heap_adjust* adj)
 #endif
 }
 
+/* About relocation of fdefns - suppose we had originally intended to load:
+ *   [ range of adjustable addresses  ]
+ *   | fixedobj space | varyobj space |
+ *          fdefn --------> callee
+ *   0x1000000        0x2000000
+ * and the subspaces were actually placed at much higher addresses
+ * that do not overlap at all with the intended addresses:
+ *                                       | fixedoboj space | varyobj space |
+ *                                       0x8000000 ...
+ * Then given an fdefn, its callee as computed from the new address appears
+ * such there would be no relocation adjustment for the addressed object.
+ * We correctly do not adjust the fdefn's callee displacement.
+ * But suppose the moved spaces overlap the expected load addresses:
+ *        [ range of adjustable addresses  ]
+ *        | fixedobj space | varyobj space |
+ *               fdefn --------> callee
+ *   0x1000000 = desired
+ *        0x1008000 = actual
+ * In that case there can be an fdefn whose callee appears to be in the
+ * adjustment range, and we'd compute an (incorrect) adjustment. In reality,
+ * both the fdefn and its intended target moved by the same amount,
+ * so the net change to the fdefn JMP instruction displacement ought to be zero.
+ * To deal with this, we have to compute the fdefn callee as if the fdefn were
+ * at its intended load address, then check whether that callee needs adjustment,
+ * and compute a net adjustment by incorporating the amount by which the fdefn
+ * itself moved.  Since both spaces move together for now, these two deltas
+ * cancel out, and the net adjustment becomes zero. The full complexity is
+ * required to handle the situation where only one of the immobile subspaces
+ * moved, which we can't actually handle yet at all.
+ * But already there was a bug when they move together by a small amount.
+ */
 static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 {
     lispobj *where = (lispobj*)start;
@@ -485,10 +511,17 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             // the offset from fdefns to code can change if:
             // - static space fdefn -> immobile space code and immobile space moved
             // - immobile spaces changed relative position
-            if ((delta = calc_adjustment(adj, fdefn_callee_lispobj((struct fdefn*)where)))) {
-                void* prel32 = 1 + (char*)(where+3);
-                FIXUP_rel(UNALIGNED_STORE32(prel32, UNALIGNED_LOAD32(prel32) + delta),
-                          prel32);
+            {
+                // See comment above this function concerning fdefn adjustment.
+                lispobj orig = inverse_adjust(adj, (lispobj)where);
+                lispobj callee = virtual_fdefn_callee_lispobj((struct fdefn*)where, orig);
+                sword_t callee_delta = calc_adjustment(adj, callee);
+                delta = callee_delta - ((lispobj)where - orig);
+                if (delta != 0) {
+                    void* prel32 = 1 + (char*)(where+3);
+                    FIXUP_rel(UNALIGNED_STORE32(prel32, UNALIGNED_LOAD32(prel32) + delta),
+                              prel32);
+                }
             }
 #endif
             continue;
@@ -608,7 +641,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 
 static void relocate_heap(struct heap_adjust* adj)
 {
-    if (!lisp_startup_options.noinform || SHOW_SPACE_RELOCATION) {
+    if (!lisp_startup_options.noinform && SHOW_SPACE_RELOCATION) {
         int i;
         for (i = 0; i < adj->n_ranges; ++i)
             if (adj->range[i].delta)
@@ -660,8 +693,9 @@ set_adjustment(struct heap_adjust* adj,
 
 static void
 process_directory(int count, struct ndir_entry *entry,
-                  int fd, os_vm_offset_t file_offset, int merge_core_pages,
-                  struct heap_adjust* adj)
+                  int fd, os_vm_offset_t file_offset,
+                  int __attribute__((unused)) merge_core_pages,
+                  struct heap_adjust __attribute__((unused)) *adj)
 {
     extern void immobile_space_coreparse(uword_t,uword_t);
 
@@ -793,9 +827,13 @@ process_directory(int count, struct ndir_entry *entry,
                 addr = FIXEDOBJ_SPACE_START + FIXEDOBJ_SPACE_SIZE;
             else
 #endif
-            if (request)
+            if (request) {
                 addr = (uword_t)os_validate(sub_2gb_flag ? MOVABLE_LOW : MOVABLE,
                                             (os_vm_address_t)addr, request);
+                if (!addr) {
+                    lose("Can't allocate %p bytes for space %d", request, id);
+                }
+            }
             switch (id) {
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
             case IMMOBILE_FIXEDOBJ_CORE_SPACE_ID:
