@@ -26,13 +26,9 @@
 #include "parse.h"
 #include "vars.h"
 
-/* Almost all of this file can be skipped if we're not supporting LDB. */
-#if defined(LISP_FEATURE_SB_LDB)
-
 #include "print.h"
 #include "arch.h"
 #include "interr.h"
-#include "gc.h"
 #include "search.h"
 #include "purify.h"
 #include "globals.h"
@@ -41,7 +37,7 @@
 #include "thread.h"
 #include "genesis/static-symbols.h"
 #include "genesis/primitive-objects.h"
-
+#include "gc-internal.h"
 
 
 /* When we need to do command input, we use this stream, which is not
@@ -61,7 +57,7 @@ typedef void cmd(char **ptr);
 
 static cmd dump_cmd, print_cmd, quit_cmd, help_cmd;
 static cmd flush_cmd, search_cmd, regs_cmd, exit_cmd;
-static cmd print_context_cmd;
+static cmd print_context_cmd, pte_cmd;
 static cmd backtrace_cmd, purify_cmd, catchers_cmd;
 static cmd grab_sigs_cmd;
 static cmd kill_cmd;
@@ -88,6 +84,7 @@ static struct cmd {
     {"purify", "Purify. (Caveat purifier!)", purify_cmd},
     {"print", "Print object at ADDRESS.", print_cmd},
     {"p", "(an alias for print)", print_cmd},
+    {"pte", "Page table entry for address", pte_cmd},
     {"quit", "Quit.", quit_cmd},
     {"regs", "Display current Lisp registers.", regs_cmd},
     {"search", "Search for TYPE starting at ADDRESS for a max of COUNT words.", search_cmd},
@@ -114,12 +111,17 @@ dump_cmd(char **ptr)
 
     char *addr = lastaddr;
     int count = lastcount, displacement;
-    int force = 0;
+    int force = 0, decode = 0;
 
     if (more_p(ptr)) {
+        // you can't both "force" and "decode" - only one or the other,
+        // or neither
         if (!strncmp(*ptr, "-f ", 3)) {
           force = 1;
           *ptr += 3;
+        } else if (!strncmp(*ptr, "-d ", 3)) {
+          decode = 1;
+          *ptr += 3;
         }
         addr = parse_addr(ptr, !force);
 
@@ -141,6 +143,13 @@ dump_cmd(char **ptr)
         count = -count;
     }
 
+    boolean aligned = ((uword_t)addr & LOWTAG_MASK) == 0;
+    if (decode && (!aligned || displacement < 0)) {
+        printf("Sorry, can only decode if aligned and stepping forward\n");
+        decode = 0;
+    }
+    lispobj* next_object = decode ? (lispobj*)addr : 0;
+
     while (count-- > 0) {
 #ifndef LISP_FEATURE_ALPHA
         printf("%p: ", (os_vm_address_t) addr);
@@ -156,7 +165,7 @@ dump_cmd(char **ptr)
             unsigned char *cptr = (unsigned char *)addr;
 
 #if N_WORD_BYTES == 8
-            printf("0x%016lx | %c%c%c%c%c%c%c%c\n",
+            printf("0x%016lx | %c%c%c%c%c%c%c%c",
                    lptr[0],
                    visible(cptr[0]), visible(cptr[1]),
                    visible(cptr[2]), visible(cptr[3]),
@@ -167,12 +176,36 @@ dump_cmd(char **ptr)
             printf("0x%08lx   0x%04x 0x%04x   "
                    "0x%02x 0x%02x 0x%02x 0x%02x    "
                    "%c%c"
-                   "%c%c\n",
+                   "%c%c",
                    lptr[0], sptr[0], sptr[1],
                    cptr[0], cptr[1], cptr[2], cptr[3],
                    visible(cptr[0]), visible(cptr[1]),
                    visible(cptr[2]), visible(cptr[3]));
 #endif
+#ifdef LISP_FEATURE_GENCGC
+            if (aligned) {
+                lispobj ptr = *(lispobj*)addr;
+                int gen;
+                if (is_lisp_pointer(ptr) && gc_managed_heap_space_p(ptr)
+                    && (gen = gc_gen_of(ptr, 99)) != 99) // say that static is 99
+                    if (gen != 99) printf(" | %d", gen);
+            }
+#endif
+            if (decode && addr == (char*)next_object) {
+                lispobj word = *addr;
+                // ensure validity of widetag because crashing with
+                // "no size function" would be worse than doing nothing
+                if (word != 0 && !is_lisp_pointer(word)
+                    && valid_widetag_p(widetag_of(word))) {
+                    printf(" %s", widetag_names[widetag_of(word)>>2]);
+                    next_object += sizetab[widetag_of(word)](next_object);
+                } else if (is_cons_half(word)) {
+                    next_object += 2;
+                } else { // disable decoder if weirdness observed
+                    decode = 0;
+                }
+            }
+            printf("\n");
         }
         else
             printf("invalid Lisp-level address\n");
@@ -191,6 +224,13 @@ print_cmd(char **ptr)
     print(obj);
 }
 
+static void
+pte_cmd(char **ptr)
+{
+    extern void gc_show_pte(lispobj);
+    gc_show_pte(parse_lispobj(ptr));
+}
+
 static void
 kill_cmd(char **ptr)
 {
@@ -200,7 +240,7 @@ kill_cmd(char **ptr)
 }
 
 static void
-regs_cmd(char **ptr)
+regs_cmd(char __attribute__((unused)) **ptr)
 {
     struct thread __attribute__((unused)) *thread=arch_os_get_current_thread();
 
@@ -219,14 +259,12 @@ regs_cmd(char **ptr)
 #ifdef LISP_FEATURE_GENCGC
     /* printf("DYNAMIC\t=\t%p\n", (void*)DYNAMIC_SPACE_START); */
 #else
-    printf("STATIC\t=\t%p   ",
-           (void*)SymbolValue(STATIC_SPACE_FREE_POINTER, thread));
-    printf("RDONLY\t=\t%p   ",
-           (void*)SymbolValue(READ_ONLY_SPACE_FREE_POINTER, thread));
+    printf("STATIC\t=\t%p   ", static_space_free_pointer);
+    printf("RDONLY\t=\t%p   ", read_only_space_free_pointer);
     printf("DYNAMIC\t=\t%p\n", (void*)current_dynamic_space);
 #endif
 
-#ifdef reg_ALLOC
+#ifndef ALLOCATION_POINTER
     printf("ALLOC\t=\t%p\n", (void*)dynamic_space_free_pointer);
 #else
     printf("ALLOC\t=\t%p\n", (void*)SymbolValue(ALLOCATION_POINTER, thread));
@@ -303,13 +341,13 @@ search_cmd(char **ptr)
  * it.) */
 
 static void
-flush_cmd(char **ptr)
+flush_cmd(char __attribute__((unused)) **ptr)
 {
     flush_vars();
 }
 
 static void
-quit_cmd(char **ptr)
+quit_cmd(char __attribute__((unused)) **ptr)
 {
     char buf[10];
 
@@ -325,7 +363,7 @@ quit_cmd(char **ptr)
 }
 
 static void
-help_cmd(char **ptr)
+help_cmd(char __attribute__((unused)) **ptr)
 {
     struct cmd *cmd;
 
@@ -337,13 +375,13 @@ help_cmd(char **ptr)
 static int done;
 
 static void
-exit_cmd(char **ptr)
+exit_cmd(char __attribute__((unused)) **ptr)
 {
     done = 1;
 }
 
 static void
-purify_cmd(char **ptr)
+purify_cmd(char __attribute__((unused)) **ptr)
 {
     purify(NIL, NIL);
 }
@@ -361,12 +399,14 @@ print_context(os_context_t *context)
         brief_print((lispobj)(*os_context_register_addr(context,i)));
 #endif
     }
-#ifdef LISP_FEATURE_DARWIN
+#if defined(LISP_FEATURE_DARWIN) && defined(LISP_FEATURE_PPC)
     printf("DAR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 41)));
     printf("DSISR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 42)));
 #endif
+#ifndef REG_PC
     printf("PC:\t\t  0x%08lx\n",
            (unsigned long)(*os_context_pc_addr(context)));
+#endif
 }
 
 static void
@@ -375,7 +415,7 @@ print_context_cmd(char **ptr)
     int free_ici;
     struct thread *thread=arch_os_get_current_thread();
 
-    free_ici = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,thread));
+    free_ici = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
     if (more_p(ptr)) {
         int index;
@@ -385,7 +425,7 @@ print_context_cmd(char **ptr)
         if ((index >= 0) && (index < free_ici)) {
             printf("There are %d interrupt contexts.\n", free_ici);
             printf("printing context %d\n", index);
-            print_context(thread->interrupt_contexts[index]);
+            print_context(nth_interrupt_context(index, thread));
         } else {
             printf("There aren't that many/few contexts.\n");
             printf("There are %d interrupt contexts.\n", free_ici);
@@ -396,7 +436,7 @@ print_context_cmd(char **ptr)
         else {
             printf("There are %d interrupt contexts.\n", free_ici);
             printf("printing context %d\n", free_ici - 1);
-            print_context(thread->interrupt_contexts[free_ici - 1]);
+            print_context(nth_interrupt_context(free_ici - 1, thread));
         }
     }
 }
@@ -417,12 +457,10 @@ backtrace_cmd(char **ptr)
 }
 
 static void
-catchers_cmd(char **ptr)
+catchers_cmd(char __attribute__((unused)) **ptr)
 {
-    struct catch_block *catch;
-    struct thread *thread=arch_os_get_current_thread();
-
-    catch = (struct catch_block *)SymbolValue(CURRENT_CATCH_BLOCK,thread);
+    struct catch_block *catch = (struct catch_block *)
+        read_TLS(CURRENT_CATCH_BLOCK, arch_os_get_current_thread());
 
     if (catch == NULL)
         printf("There are no active catchers!\n");
@@ -447,7 +485,7 @@ catchers_cmd(char **ptr)
 }
 
 static void
-grab_sigs_cmd(char **ptr)
+grab_sigs_cmd(char __attribute__((unused)) **ptr)
 {
     extern void sigint_init(void);
 
@@ -536,20 +574,9 @@ throw_to_monitor()
     longjmp(curbuf, 1);
 }
 
-#endif /* defined(LISP_FEATURE_SB_LDB) */
-
 /* what we do when things go badly wrong at a low level */
 void
 monitor_or_something()
 {
-#if defined(LISP_FEATURE_SB_LDB)
     ldb_monitor();
-#else
-     fprintf(stderr,
-"The system is too badly corrupted or confused to continue at the Lisp\n\
-level. If the system had been compiled with the SB-LDB feature, we'd drop\n\
-into the LDB low-level debugger now. But there's no LDB in this build, so\n\
-we can't really do anything but just exit, sorry.\n");
-    exit(1);
-#endif
 }
