@@ -29,6 +29,7 @@
 #include "interr.h"
 #include "gc.h"
 #include "gc-internal.h"
+#include "gc-private.h"
 #include "thread.h"
 #include "genesis/gc-tables.h"
 #include "genesis/primitive-objects.h"
@@ -36,7 +37,6 @@
 #include "genesis/layout.h"
 #include "genesis/defstruct-description.h"
 #include "genesis/hash-table.h"
-#include "gencgc.h"
 
 /* We don't ever do purification with GENCGC as of 1.0.5.*. There was
  * a lot of hairy and fragile ifdeffage in here to support purify on
@@ -248,8 +248,9 @@ static lispobj
 ptrans_code(lispobj thing)
 {
     struct code *code = (struct code *)native_pointer(thing);
-    long nwords = code_header_words(code->header)
-                + code_instruction_words(code->code_size);
+    long nwords =
+        ALIGN_UP(code_header_words(code->header) + code_unboxed_nwords(code->code_size),
+                 2);
 
     struct code *new = (struct code *)newspace_alloc(nwords,1); /* constant */
 
@@ -352,7 +353,7 @@ ptrans_returnpc(lispobj thing, lispobj header)
     return new + (thing - code);
 }
 
-#define WORDS_PER_CONS CEILING(sizeof(struct cons) / sizeof(lispobj), 2)
+#define WORDS_PER_CONS ALIGN_UP(sizeof(struct cons) / sizeof(lispobj), 2)
 
 static lispobj
 ptrans_list(lispobj thing, boolean constant)
@@ -377,7 +378,7 @@ ptrans_list(lispobj thing, boolean constant)
 
         /* And count this cell. */
         length++;
-    } while (lowtag_of(thing) == LIST_POINTER_LOWTAG &&
+    } while (listp(thing) &&
              dynamic_pointer_p(thing) &&
              !(forwarding_pointer_p(*native_pointer(thing))));
 
@@ -529,15 +530,16 @@ pscav(lispobj *addr, long nwords, boolean constant)
 #endif
               case SAP_WIDETAG:
                 /* It's an unboxed simple object. */
-                count = CEILING(HeaderValue(thing)+1, 2);
+                count = ALIGN_UP(HeaderValue(thing)+1, 2);
                 break;
 
               case SIMPLE_VECTOR_WIDETAG:
-                  if (HeaderValue(thing) == subtype_VectorValidHashing) {
-                    struct hash_table *hash_table =
-                        (struct hash_table *)native_pointer(addr[2]);
-                    hash_table->needs_rehash_p = T;
-                  }
+                // addr[0] : header
+                //     [1] : vector length
+                //     [2] : element[0] = hashtable
+                //     [3] : element[1] = rehash bit
+                if (is_vector_subtype(thing, VectorValidHashing))
+                    addr[3] = make_fixnum(1);
                 count = 2;
                 break;
 
@@ -572,7 +574,7 @@ pscav(lispobj *addr, long nwords, boolean constant)
                     long nslots = instance_length(*addr) | 1;
                     int index;
                     if (fixnump(lbitmap)) {
-                      sword_t bitmap = (sword_t)lbitmap >> N_FIXNUM_TAG_BITS;
+                      sword_t bitmap = fixnum_value(lbitmap);
                       for (index = 0; index < nslots ; index++, bitmap >>= 1)
                         if (bitmap & 1)
                           pscav(slots + index, 1, constant);
@@ -764,7 +766,8 @@ purify(lispobj static_roots, lispobj read_only_roots)
 }
 #else /* LISP_FEATURE_GENCGC */
 int
-purify(lispobj static_roots, lispobj read_only_roots)
+purify(lispobj __attribute__((unused)) static_roots,
+       lispobj __attribute__((unused)) read_only_roots)
 {
     lose("purify called for GENCGC. This should not happen.");
 }
